#!/bin/csh

limit memory    unlimited
limit datasize  unlimited
limit stacksize unlimited


#############################################################################
#                                                                           #
#  CYCLE-YPR2X.COM  -  2-FOLD PROPER CYCLIC AVERAGING WITH MAIN             #
#                                                                           #
#                                                           LAD   15-04-96  #
#                                                                           #
#############################################################################


set DATE          =  "110896"

set TEMP_DIR      =  ${tmp}/aver
set RMS_DIR       =  ${RMS_DIR}/prop

set FOB           =  $YXD/fob/yprpre3t1a-22.fob

set MODEL         =  $YXD/yprpre3t1a-posb-${DATE}.xpl

set MAP_IN        =  ${TEMP_DIR}/yprpre3t1a-2fofc-${DATE}.xmap
set MAP_OUT       =  ${TEMP_DIR}/yprpre3t1a-2fofc-10cy-pr-${DATE}.rmap
set MAP_1         =  ${TEMP_DIR}/yprpre3t1a-2fofc-1cy-pr-${DATE}.rmap

set RESO_LOW      =  "50.0"                        ## for back transformation
set RESO_HIGH     =  "2.4"
set SYMM          =  ">symm/p21.symm"              ## symmetry of space group
set FOUR_COEFF    =  "fobs * 2 - fcalc * 1"           ## fourier coefficients
set MAX_CYCLES    =  10                  ## number of cycles for each par.set
set LIMITS        =  "0  0  -70"

#############################################################################

set SUM           =  cycle-2x.sum       
set ATOM_SEL      =  "all"
set MASK_CUSHION  =  "20"               ## external boundaries in grid points
set MASK_RADIUS   =  "3.5"                        ## mask radius in Angstroms
set MAINV         =  "/kaefer/u08/turk/main/main_mega_97.exe_UALP"     
set LEVEL         =  5
set MAP           =  ${TEMP_DIR}/cycle-pr.rmap                ## internal map
set MASK          =  ${TEMP_DIR}/mask-pr.rmap                 ## internal map
set R_SHELLS      =  10                         ## r-factor resolution shells
set CHKP          =  `date +"%H %M %S"`
set EXIT          =  ""
set CYCLE         =  1

#############################################################################

echo ""
echo ""
echo "      ***  PROPER CYCLIC AVERAGING WITH MAIN  ***"
echo ""
echo "                                           host:  $HOST"
echo "                                           type:  $HOSTTYPE    os:  $OS"
echo "                                           user:  $LOGNAME"
echo -n "                                           date:  ";date
echo ""
echo ""

#############################################################################
#

printf "\n\n           ***  SUMMARY OF PROPER CYCLIC AVERAGING WITH MAIN  ***\n\n\n\n" >! ${SUM}

printf "* %66s%18.2f A\n" "RESOLUTION:" ${RESO_HIGH} >>! ${SUM}

printf "----------------------------------------------------------------------------------------\n" >>! ${SUM}

#---------------
###goto back
#---------------

#############################################################################
#
#  convert input map from xpl to numb
#

${MAINV} << eof_main

 set level ${LEVEL}
 read file ${MAP_IN} map xpl
 sh map
 read file ${SYMM} symm
 write over file ${MAP} map 1 numb

 quit

eof_main


#############################################################################
#
# generate mask from model
#
mask:

${MAINV} << eof_main

 read file ${MODEL} coor xpl
 set level ${LEVEL}
 read file ${MAP} map numb  
 read file ${SYMM} symm

 key SEL sele ${ATOM_SEL} end

 make map 2 from 1 init -9999 around ${MASK_CUSHION} sele SEL end
 sh map
 make map 2 atom mask ${MASK_RADIUS} sele SEL end
 sh map
 write over file ${MASK} map 2 numb
 
 quit

eof_main


#############################################################################
#
# averaging
#

aver:

set CYCLE = 1
printf "* PROCESSED CYCLES:            " >>! ${SUM}


############################ LOOP_I #########################################
loop_i:

printf "${CYCLE}  " >>! ${SUM}

printf "\n##############################################################################\n"
printf "#### MAIN: averaging - cycle * ${CYCLE} * ####\n\n"

#############################################################################

2fold:

${MAINV} << eof_main

 set level ${LEVEL}
 set vari MAT_ROT int = 8
 read file ${MAP} map numb  
 read file ${MASK} map numb
 read file ${SYMM} symm

 make map 2 from 1 copy 
 make map 3 from 2 init 9999 copy
 show map
 make map 2 set -9990 10000 9999                   ! regeneration of mask
 show map

 @${RMS_DIR}/ypr-2halves.com
 make map 2 from 1 rota matr MAT_ROT  -
          tran vect XTRAN YTRAN ZTRAN -
          center coor 0. 0. 0.

 make map 3 from 2 add
! make map 2 set -9990 10000 9999

 show map
 make map 3 rescale

 write over file ${TEMP_DIR}/aver.rmap map 3 numb

 quit

eof_main


#############################################################################
#
# backtransformation
# map calculation
#

back:

printf "\n##############################################################################\n"
printf "#### MAIN: backtransformation and calculation of new map - cycle * ${CYCLE} *  ####\n\n"

${MAINV} << eof_main >&! ${TEMP_DIR}/backmain.log

 read file ${TEMP_DIR}/aver.rmap map numb
 read file ${SYMM} symm
 make map 2 from 1 init -9999 cell 
 make map 2 from 1 cell
 sh map

 make map 2 set -10000 -9900 0.0 
 sh map

 read file ${FOB}  -
          reflect init resol  -
     ${RESO_HIGH} ${RESO_LOW} -
     limit ${LIMITS}

 set level ${LEVEL}

 fourier map 2
 reflect shell ${R_SHELLS} r-values
 reflect set ampl phase fwork = fcalc * 1
 reflect set ampl scale fcalc fwork = ${FOUR_COEFF}

 make map 2 zero
 make map 2 convert complex
 reflect fill-map 2 sele defined end
 fourier map 2 back
 make map 2 rescale
 sh map

 set level 0
 read file ${MODEL} coor xpl
 set level ${LEVEL}
 key SEL sele ${ATOM_SEL} end

 make map 3 from 2 init 9999  around ${MASK_CUSHION} sele SEL end
 make map 3 from 2 copy
 sh map

 write over file ${MAP} map 3 numb

 quit

eof_main

cat ${TEMP_DIR}/backmain.log

#############################################################################

if ( ${CYCLE} == "1") then
  \cp ${MAP} ${MAP_1}
endif

#############################################################################
#
# goto next aver cycle
#

@ CYCLE++

if (${CYCLE} <= ${MAX_CYCLES}) goto loop_i

printf "\n" >>! ${SUM}

nawk 'BEGIN {last=0} {if ($0 ~ /shell  resolution range/) last=NR+10; if (NR == last) print $0}' < ${TEMP_DIR}/backmain.log >>! ${SUM}

\cp ${MAP} ${MAP_OUT}

#############################################################################

time:

time >! ${TEMP_DIR}/time.tmp
set TIME = `cat ${TEMP_DIR}/time.tmp`
set CHKP_OLD = `echo ${CHKP}`
set CHKP =  `date +"%H %M %S"`

set CYC_TIME = "`bc << eof_bc \
s = ((${CHKP[1]}*3600+${CHKP[2]}*60+${CHKP[3]}) - (${CHKP_OLD[1]}*3600+${CHKP_OLD[2]}*60+${CHKP_OLD[3]}))  \
m = s/60   \
s = s%60   \
m;s        \
eof_bc`"

printf "%66s:%12s (%3d:%02d)\n" "TIME" ${TIME[1]} ${CYC_TIME[1]} ${CYC_TIME[2]} >>! ${SUM}

printf "----------------------------------------------------------------------------------------\n" >>! ${SUM}

#############################################################################

end:

exit
