! Read an MIR map, symmetry operations and a reflection file.

read file ../model/mir_clear_4.0_heavy.xmap map xplor
read file >symm/c2.symm symm
read file catl.fobs refl init limit -99 0 0 reso 3.5 100.0

! Set variables MAP_DENS (origin of density) and MAP_SOLV (solvent
! falttened map).

set vari MAP_DENS = 1
set vari MAP_SOLV = 2
set vari SOLV_CUT = 0.4
set vari SOLV_RAD = 8.0

! Initiate a unit cell with the density set to EMPTY and a density of an
! atom that is placed at coordinate system origin and which density is
! distance dependent (1. - d / SOLV_RAD).  Set the empty points to zero,
! Fourier transform the map and save its Fourier coeficients in
! the FWORKSET array.

make map MAP_SOLV from MAP_DENS init -9999  cell real
make map MAP_SOLV atom dist SOLV_RAD
make map MAP_SOLV set -100000 0 0.0
fourier map MAP_SOLV
reflect set ampl phase fwork = fcalc * 1.0

! Copy the MIR density into the new MAP_SOLV, set all negative density
! values to zero and Fourier transform it.  R-value calculation is
! performed in order to follow convergence of a further procedure.

make map MAP_SOLV from 2 init 9999  cell real
make map MAP_SOLV from MAP_DENS copy
make map MAP_SOLV set -99999 0 0.0
fourier map MAP_SOLV
reflect shells 10 r-values 

! Multiply FWORKSET (sphere model) and FCALCULATE (truncated MIR
! density) Fourier coeficients, convert the MAP_SOLV map to a convex
! map, fill it with the multiplied Fourier coefficients and Fourier BACK
! transform it.

reflect set ampl phase fwork = fwork * 1.0  *  fcalc * 1.0
make map MAP_SOLV conv complex
make map MAP_SOLV set -9999999 9999999 0.0
refl fill-map MAP_SOLV symm 1 1 
four map MAP_SOLV back 
make map MAP_SOLV rescale 

! Perform the score map analyses (map occupancy in 256 steps), convert
! the selected parts to a MASK region and copy density into it.  Set the
! density of the empty region to, Fourier transform it and caluclate
! R-values.

anal ini
anal map MAP_SOLV
make map MAP_SOLV anal SOLV_CUT
make map MAP_SOLV from MAP_DENS copy
make map MAP_SOLV set -99999 -9000 0.0
fourier map MAP_SOLV
reflect shells 10 r-values 

! Save the FCALCULATE values in the FWORKSET array and calculate
! an Fobs map 

reflect set ampl phase fwork = fcalc * 1.0
reflect set ampl fwork = fobs * 1.0

! or 2 Fobs - Fcalc map.

! reflect set ampl scale fcalc fwork = fobs * 2.0 - fcalc * 1.0


make map MAP_SOLV set -9999999 9999999 0.0
refl fill-map MAP_SOLV symm 1 2 defined 
fourier back map MAP_SOLV
make map MAP_SOLV rescale

return
quit

Here you can either save or display the solvent flattened
map or calculated the next cycle by redefining the variables:

> set vari MAP_DENS = 2
> set vari MAP_SOLV = 3

