      PROGRAM MAPAGE
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   Program MAPAGE is used to read in an electron density map and produce a 
C   DSN6 style electron density map for the on-the-fly contouring routine in 
C   program PS300 FRODO.
C
C   The input Fourier map of real-space densities, sampled on a grid aligned 
C   with the crystal unit cell axes, is converted into the particular direct
C   access form used by PS300 FRODO as a DSN6 input file. FRODO uses this 
C   file under its MAP option. 
C
C   MAPAGE proceeds in two passes through the input file.  It first reads 
C   through the file to obtain the values of PROD and IPLUS.  These values are 
C   used to scale the input electron density map values into byte elements 
C   between 0 and 255.  It determines the appropriate values of PROD and IPLUS
C   to take full advantage of the 0-255 dynamic range of the file. 
C
C   As written, the program requires the input electron density map to be
C   in Steigemann's PROTEIN Style map file with format-5 records. To use 
C   MAPAGE with other electron density map formats than found in Steigemann's 
C   PROTEIN program requires changing subroutine MAPIN.
C
C   The original VAX version written by Alywn Jones (27-JUN-81) and modified
C   modified by Bruce Bush (Merck & CO)  (17-NOV-82).
C
C   Changes for version 6.3:
C
C   Save the input and output file names, and print them in the listing file.
C   Give error message if unable to open input file
C
C   Correct a problem that occured when using a map with an odd number of
C   points in a row. 
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER (MAXRHO = 32000 )
      PARAMETER ( LUN1 = 1,  LUN2 = 2,  LUN5 = 5,  LUN6 = 6 )
	parameter (nrmaps=7000000)
	parameter (nbmaps=80)
	dimension  bmaps(nbmaps), rmaps(nrmaps)
	character*80 line
	character*1 bmaps
	dimension rhodat(4), index(3)
	real*8 dsigma
      COMMON /MAPIN1/ IOXYZ(3), NXYZ(3), IUVW(3),  IGRID(3), CELL(6)
      COMMON /MAPIN2/ RHORNG(6), NRHO 
      COMMON /MAPIN3/ PROD,  IPLUS, IHIST(256)
	common /symm/ nsymm, rts(12,60)
      CHARACTER*30 FILEI, FILEO
      INTEGER      FIRST(256), SLICE(8,MAXRHO), RECORD(512) ,IXYZ(3)
      INTEGER      NBXYZ(3), NUVW(3), NBUVW(3), IBXYZ(3)
      REAL         RHO(MAXRHO) 
C
      WRITE (LUN6,1000)
C
C   Open the input electron density file 
C
c	lun1 = 1
	ires = 2
	write(*,*) ' PROTEIN MAP FILE NAME:'
	read(*,99,err=900) (line(j:j),j=1,80)
99	format( 80a1 )
	filei(1:30) = line(1:30)
	open ( unit = lun1, file = line, shared, readonly,
     * form='unformatted',maxrec=2044, status = 'old', err = 900 )
c      WRITE (LUN6,1100)
c      READ  (LUN5,1200) FILEI
c      CALL ADDEXT (FILEI,'FFT')
c      CALL FOPEN  (LUN1,FILEI,5,.FALSE.,IERR)
c      IF (IERR.NE.0) GOTO 900
C
C   Open the output random access MAP file
C
	write(*,*) ' MAIN MAP FILE NAME:'
	read(*,99,err=900) (line(j:j),j=1,80)
	open ( unit = ires, file = line, status = 'new',
     * carriagecontrol = 'list', err = 910 )
c      WRITE (LUN6,1300)
c      READ  (LUN5,1200) FILEO
c      CALL ADDEXT (FILEO,'DN6')
c      CALL FOPEN  (LUN2,FILEO,4,.TRUE.,IERR)
c      IF (IERR.NE.0) GOTO 910
C
C   Write header for output listing
C
      WRITE (LUN6,1400) 
      WRITE (LUN6,1500) FILEI
      WRITE (LUN6,1600) FILEO
C
C   Read in map header 
C
      CALL MAPIN (.TRUE.,1,RHO,1,IZ,IERR)
      IF (IERR.NE.0) STOP
      WRITE (LUN6,1700) CELL,IOXYZ,NXYZ,IGRID,IUVW
C
C   Number of points along U, V, and W
C
      NUVW(1)   = NXYZ(IUVW(1))
      NUVW(2)   = NXYZ(IUVW(2))
      NUVW(3)   = NXYZ(IUVW(3))
C
C   Number of bricks along U, V, and W
C
      NBUVW(1)  = (NUVW(1)+7)/8
      NBUVW(2)  = (NUVW(2)+7)/8
      NBUVW(3)  = (NUVW(3)+7)/8
C
C   Number of Bricks along X, Y, and Z
C
      NBXYZ(1)  = (NXYZ(1)+7)/8
      NBXYZ(2)  = (NXYZ(2)+7)/8
      NBXYZ(3)  = (NXYZ(3)+7)/8
      WRITE (LUN6,1800) NBXYZ
C
C   Number of points in a map section
C
      LAYER     = NUVW(1) * NUVW(2)
      IF (LAYER.GT.MAXRHO) GOTO 920
C
C   Read through map to calculate scale factors
C
	do i = 1,3
	if ( iuvw(i) .eq. 1 ) ixyz(1) = i
	if ( iuvw(i) .eq. 2 ) ixyz(2) = i
	if ( iuvw(i) .eq. 3 ) ixyz(3) = i
	end do
c
	write(*,*) ' nuvw', nuvw
	do 100 i  = 1, nuvw(3)
	CALL MAPIN (.FALSE.,1,rho,MAXRHO,IZ,IERR)
	IF (IERR.NE.0) GOTO 930
	index(3) = i
	do 5 k = 1, nuvw(2)
	index(2) = k
	do 5 j = 1, nuvw(1)
	index(1) = j
	nn = index(iuvw(1)) + nxyz(1)*(index(iuvw(2))-1) +
     * nxyz(2)*nxyz(1)*(index(iuvw(3))-1)
	rmaps(nn) = rho( (k-1)*nuvw(1) + j)
5	continue
100	continue
c  
	nmaps = layer * nuvw(3)
	rmin =  99999999.9
	rmax = -99999999.9
	aver = 0.0
	do 2 i = 1, nmaps
	aver = aver + rmaps(i)/float( nmaps)
	if ( rmaps(i) .gt. rmax ) rmax = rmaps(i)
	if ( rmaps(i) .lt. rmin ) rmin = rmaps(i)
2	continue
	delta = ( rmax - rmin ) / 255
	write(*,*)  ' MINIMUM, MAXIMUM AND DIFFERENCE STEP IN DENSITY ', 
     * rmin, rmax, delta
c
	dsigma = 0.0
	do 4 i = 1, nmaps
	rms = aver - rmaps(i)
	dsigma = dsigma + rms*rms
4	continue
	sigma = dsqrt(dsigma/ dfloat(nmaps-1))
	write(*,*) ' SIGMA: ', sigma
	open ( unit = ires, file = line, status = 'new',
     * carriagecontrol = 'list', err = 104 )
c
12	format( 6e12.5 )
111	format( 10i5 )
	write(ires,12) cell
	write(ires, 111) igrid, ioxyz, nxyz, nsymm
	if ( nsymm .gt. 0 ) then
	write(ires, 89) (j,((rts(3*k+i,j),k=1,3),i=1,3), 
     * (rts(i,j),i=1,3), j=1,nsymm)
c	write(ires, 89) (j,(rts(i,j),i=1,12), j=1,nsymm)
89	format( i5, 3f10.3,/,5x, 3f10.3,/,5x,3f10.3,5x,3f10.3)
	end if
	write(ires,12) rmin/sigma, rmax/sigma, delta/sigma
c	write(0,*) 'i, inte, del, del/delta, ichar(bmaps(i)), '
	ii = 0
	do 3 i = 1, nmaps
	if ( ii .ge. 80 ) then
	write(ires, 44, err=104) (bmaps(j), j=1, 80)
	ii = 0
	end if
	ii = ii + 1
	inte = ifix( ( rmaps(i) - rmin ) / delta + 0.5)
	bmaps(ii) = char(inte)
c	del = rmaps(i)-rmin
c	write(0,*) i, inte, del, del/delta, ichar(bmaps(i))
3	continue	
44	format(80a1)
	if ( ii .gt. 0 ) write(ires, 44, err=104) (bmaps(i), i=1, ii)
	stop
104	write(*,*) ' ERROR WRITEING MAIN DENSITY FILE'
	stop
C
C   Errors
C
  900 WRITE (LUN6,2000) FILEI
      STOP
C
  910 WRITE (LUN6,2100) FILEO
      STOP
C
  920 WRITE (LUN6,2200) (NUVW(I),I=1,2), LAYER, MAXRHO
      STOP
C
  930 WRITE (LUN6,2300) IERR, I
      STOP
C
 1000 FORMAT ('0MAP> Program MAPAGE (PS300 FRODO version 6.3)',//,
     1        ' MAP> RING 3-D map program for on-the-fly contouring')
 1100 FORMAT ('$MAP> Enter the name of the input Fourier file : ')
 1200 FORMAT (A)
 1300 FORMAT ('$MAP> Enter the name of the output DSN6 file : ')
C
 1400 FORMAT ('0MAP> Program MAPAGE (PS300 FRODO version 6.3)')
 1500 FORMAT ('0MAP> Electron density from Fourier file : ',A)
 1600 FORMAT ('0MAP> Electron density written to DSN6 file : ',A)
 1700 FORMAT ('0MAP> Fourier parameters as read from the data-set'//
     1        ' MAP> Cell : ',6F10.5,/,' MAP> Origin : ',5X,3I5,/,
     2        ' MAP> Extent : ',5X,3I5,/, ' MAP> Grid :   ',5X,3I5,/,
     3        ' MAP> U=XYZ(',I1,')   V=XYZ(',I1,')    W=XYZ(',I1,')')
 1800 FORMAT (' MAP> Number of pages along X=',I3,'   Y=',I3,'  Z=',I3)
 1900 FORMAT (' MAP> Input map values scaled to range of 0-255 by',/,
     1        ' MAP> New value = Old value * ',E10.3,' + ',I10)
 2000 FORMAT (' MAP> ERROR opening input file ',A)
 2100 FORMAT (' MAP> ERROR opening output file ',A)
 2200 FORMAT (' MAP> Layer is ',I6,' by ',I6,'=',I10,' total points',/,
     1        ' MAP> Maximum limit is ',I6,' points')
 2300 FORMAT (' MAP> ERROR ',I2,' reading map file on layer',I5)
c      RHODIF    = RHORNG(2)-RHORNG(1)
c      PROD      = 0.0
c      IF (RHODIF.GT.0.0) PROD = 250./RHODIF
c      IPLUS     = 3 - (PROD*RHORNG(1))
c      WRITE (LUN6,1900) PROD,IPLUS
cC
cC   Write output file header block 
cC
cc      CALL PUTDN6 (LUN2,1,FIRST,IERR)
c      IF (IERR.NE.0) STOP
cC
cC   Clear SLICE array
cC
c      DO 110 J = 1,MAXRHO
c      DO 110 I = 1,8
c      SLICE(I,J) = 0.0
c  110 CONTINUE
cC
cC   Reopen the map file      
cC
c      CALL MAPIN (.TRUE., 1, RHO, MAXRHO, IZ, IERR)
c      IF (IERR.NE.0) GOTO 930
cC
cC   Read in 8 layers at a time and send scaled values to output file
cC
c      IBXYZ(IUVW(3)) = 0
c      DO 150 IWBOX = 1,NUVW(3)
c      I1        = MOD(IWBOX-1,8)+1
cC
c      CALL MAPIN(.FALSE.,1,RHO,MAXRHO,IZ,IERR)
c      IF (IERR.NE.0) GOTO 930
c      DO 120 IUV = 1, LAYER
c      IVALUE    = RHO(IUV)*PROD+IPLUS
c      IF (IVALUE.GT.255) IVALUE    = 255
c      IF (IVALUE.LT.  0) IVALUE    = 0
c      IHIST(IVALUE+1) = IHIST(IVALUE+1) + 1
c      SLICE(I1,IUV)   = IVALUE
c  120 CONTINUE
cC
cC   After every eight levels, write output record
cC
c      IF (I1 .EQ. 8 .OR. IWBOX .EQ. NUVW(3)) THEN
cC
c      IF (I1.NE.8) THEN
c       DO 125 IUV = 1, LAYER
c       DO 125 I2  = I1+1,8
c       SLICE(I2,IUV)   = IPLUS
c  125  CONTINUE
c      END IF
cC
cC   Break into 8 x 8 x 8 boxes
cC
c       IBXYZ(IUVW(3)) = IBXYZ(IUVW(3)) + 1
c       DO 140 IVBOX  = 1, NBUVW(2)
c       IBXYZ(IUVW(2)) = IVBOX
c        IVBOX1       = (IVBOX-1) * 8 + 1
c        IVBOX2       = (IVBOX  ) * 8
c       DO 140 IUBOX  = 1, NBUVW(1)
c       IBXYZ(IUVW(1)) = IUBOX
c        IUBOX1       = (IUBOX-1) * 8 + 1
c        IUBOX2       = (IUBOX  ) * 8
cC
cC   Loop over Z, Y, and X points in current box
cC
c      ICT       = 0
c      DO 130 IW = 1,8
c      IXYZ(IUVW(3))   = IW
c      DO 130 IV = IVBOX1, IVBOX2
c      IXYZ(IUVW(2))   = IV - IVBOX1 + 1
c      DO 130 IU = IUBOX1, IUBOX2
c      IXYZ(IUVW(1))   = IU - IUBOX1 + 1
c      ICT       = ICT + 1
c      JCT        = IXYZ(1) + (IXYZ(2)-1)*8 + (IXYZ(3)-1)*64
cC
cC   If either direction over edge,pack record
cC
c      IF ( IU .GT. NUVW(1) .OR. IV .GT. NUVW(2) ) THEN
c       RECORD(JCT) = 0
c      ELSE
c       RECORD(JCT) = SLICE(IW,(IV-1) * NUVW(1) + IU)
c      END IF
c  130 CONTINUE
cC
c      INDEX      =  IBXYZ(1) + (IBXYZ(2)-1)*NBXYZ(1) 
c     1           + (IBXYZ(3)-1)*NBXYZ(1)*NBXYZ(2) + 1
cc      CALL PUTDN6 (LUN2,INDEX,RECORD,IERR)
c      IF (IERR.NE.0) STOP
c  140 CONTINUE
c      END IF
c  150 CONTINUE
cC
cC   Write density statistics:
cC
c      CALL XCSTAT( rhodat )
c	rmin = rhorng(2)/rhodat(2)
c	rmax = rhorng(2)/rhodat(2)
c	delta = (rmax - rmin) / 255
c	write(ires,12) rmin, rmax, delta
c      STOP
      END
