      SUBROUTINE MAPIN (DOOPEN,LUN,RHO,NSIZE,NZ,IERROR)
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   Subroutine MAPIN is designed to read in electron density maps from 
C   the Steigeman PROTEIN crystallographic program package having type-5 
C   records.  The routine was originally written by T.A. Jones, completely 
C   modified by B.L. Bush, and converted to INTEGER*4 format by J.S. Sack.
C
C  Variables used :
C
C  DOOPEN  Logical variable determines if the electron density map file
C          should be opened, or if the next layer should be read.
C          if .true., open the fourier map file and read header(s);
C          if .false.,read one section (layer)
C
C  LUN     Integer*4 variable giving the logical unit number of the input 
C          electron density map file.
C
C  NSIZE   Integer*4 variable giving the maximum dimension of variable RHO
C          (number of points in a layer).
C
C  RHO     Real*4 array to be filled with one section's worth of density on 
C          each call to MAPIN (with DOOPEN = .false.).
C
C  NZ      Integer*4 variable set to -1 on return from call with (DOOPEN) and
C          incremented on return from successful read of a section. 
C
C  IERROR  Integer*4 variable which returns 0 on success, nonzero on various 
C          failures. The following error codes are currently in use :
C
C          IERROR = 1  indicates error during read of electron density map
C          IERROR = 2  indicates error in the BLOCK ID for the record
C          IERROR = 3  indicates error in the FILE ID of the R0 record
C          IERROR = 4  indicates error in the unit cell parameters
C          IERROR = 5  indicates error in the map type (IPATT) of the R2 record
C          IERROR = 6  indicates error in the parameters of the R2 record
C          IERROR = 7  indicates error in the buffer size of the R5 record
C          IERROR = 8  indicates error in the number of words per record
C          IERROR = 9  indicates error in the number of lines
C          IERROR = 10 indicates error in the length of each line
C          IERROR = 11 indicates error in the NWR parameter
C
C  Common block MAPIN1 :
C
C  IOXYZ   Integer*4 array of dimension 3 which returns the "origin" of the map
C          (the row, column and layer index of the first entry of the map).
C
C  IMXYZ   Integer*4 array of dimension 3 which returns the "extent" of the map
C          (number of indices in the row, column, and layer directions).
C
C  IUVW    Integer*4 array of dimension 3 containing the orientation of the map
C          with respect to the crystallographic axes.  Most FRODO programs
C          require the map is in layer to layer order. 
C
C  IGRID  Integer*4 array of dimension 3 which returns the map "grid"
C          (number of points along the unit cell).
C
C  CELL    Real*4 array of dimension 6 containing the real space cell
C          cell parameters (in Angstoms and Degrees).
C
C  Common block MAPIN2
C
C  RHORNG  Real*4 array of dimension 6 containing (a) minimum and (b) maximum 
C          density in map file; (c) sum of density entries  sum (r), (d) sum of
C          squared entries sum (r*r), (e) sum of entries to 3d  power, and 
C          (f) sum of entries to 4th power of sum (r*r*r), sum (r*r*r*r). These
C          statistics are initialized on (DOOPEN) calls to +huge,-huge,0,0,0,0;
C          and then updated on later calls.  These values are used by the 
C          calling routines to report statistics at end.
C
C  NRHO    Integer*4 variable used to count the number of points in the map.
C          It is set 0 on (DOOPEN), and then incremented by the number of
C          entries in layer after each read.
C
C  IBP0    Integer*4 variable used internally to keep track of current record
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( LUN6 = 6 )
      COMMON /MAPIN1/ IOXYZ(3), IMXYZ(3), IUVW(3), IGRID(3), CELL(6)
      COMMON /MAPIN2/ RHORNG(6), NRHO, IBP0
      LOGICAL DOOPEN      
      INTEGER MGRID(3), MIFD(9), IPERM(3)
      REAL    RHO(NSIZE)  
C
      IERROR       = 0
C
C   First time through open file and initialize the variables
C
      IF (DOOPEN) THEN
C
      NZ        = -1
      RHORNG(1) = 10000000.0
      RHORNG(2) = -10000000.0
      RHORNG(3) = 0.0
      RHORNG(4) = 0.0
      RHORNG(5) = 0.0
      RHORNG(6) = 0.0
      NRHO      = 0
      IBP0      = 0
C
C   Rewind the input electron density file
C
      REWIND LUN
C
C   Read the PROTEN header records
C
      CALL GET012 ( LUN, CELL, NSYM, IPERM, MGRID, MIFD, IERROR)
      IF (IERROR.NE.0) RETURN
c	write (*,*)  ' CELL ', cell
c	write(*,*)   ' Nsymm', nsym
c	write(*,*) ' iperm', iperm
c	write(*,*) 'mifd ', mifd
c	write (*,*) ' err', ierror
C
C   Load in common block MAPIN1
C
      DO 100 I  = 1,3
       IPT       = (I-1)*3+1
       IOXYZ  (IPERM(I)) = MIFD(IPT)
       IMXYZ  (IPERM(I)) = MIFD(IPT+1)-MIFD(IPT)+1
       IF(MIFD(IPT+2).NE.1) GO TO 900
       IGRID (IPERM(I)) = MGRID(I)
  100 CONTINUE
      IUVW(1)   = IPERM(2)
      IUVW(2)   = IPERM(1)
      IUVW(3)   = IPERM(3)
      RETURN
      END IF
C
C   In each subsequent pass, read one layer of the map
C
      CALL GETR5 (LUN,MIFD,NSIZE,RHO,RHORNG,NRHO,IBP0,IERROR)
      IF (IERROR.NE.0) RETURN
      NZ        = NZ + 1
      RETURN
C
  900 IERROR       = 6
      WRITE (LUN6,2000) (MIFD(I),I=3,9,3)
      RETURN
C
 2000 FORMAT (' MAP> DELTA values are not all 1 (',3I3,')')
      END

      SUBROUTINE GET012 (LUN, CELL, NSYM, IPERM, IGRID, MIFD, IERROR)
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   MAPIN subroutine GET012 is used to read in the Protein style header records
C
C   Original Source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( DEGTOR = 180./3.141593 , LUN6 = 6 )
      INTEGER IPERM(3), IGRID(3), MIFD(9), TITLE(6) 
	integer*2 ip(2)
      REAL    CELL(6)
	common / symm/ nsymm, rts(12,60)
C
C   Read the type zero record
C
      READ (LUN,ERR=900) IBLID,IP,NWR,
     1     (TITLE(I),I=1,4),IFT,(TITLE(I),I=5,6)
c	write(*, '( 4Hirec, i5, 2z10, 2i5)' ) iblid, ip, nwr, ift
	write(*,88) title
88	format( ' TITLE:', 6a4)
      IF (IBLID.NE.0)    GOTO 910
      IF (IFT.NE.3)      GOTO 920
C
C   Read the type 1 record
C
c	write(*,*) ' RECORD 11111111111'
      READ (LUN,ERR=900) IBLID,IP,NWR,CELL,NSYM
c	write(*, '(  i5, z10, i5)' ) iblid, ip, nwr
c	write(*,*) ' nmr', iblid, ip, nwr
	nsymm = nsym
      IF(IBLID.NE.1) GOTO 910
	rewind(unit=lun)
	read(lun) idum
      READ (LUN,ERR=900) IBLID,IP,NWR,CELL,idum,
     * ( (rts(i,j), i=1,12), j=1,nsym)
c	write(*,89) (j,(rts(i,j),i=1,12), j=1,nsym)
c89	format( i5, 3f10.3,/,5x, 3f10.3,/,5x,3f10.3,5x,3f10.3)
C
C  Convert COS(ANGLE) to ANGLE
C
      DO 100 I  = 4,6
      IF ( ABS( CELL(I) ).GT.1.0 ) GO TO 930
      CELL(I) = ACOS(CELL(I)) * DEGTOR
  100 CONTINUE
C
C   Read the type 2 record
C
c	write(*,*) ' RECORD 222222222'
      READ (LUN,ERR=900) IBLID,IP,NWR,IPATT,IPERM,IGRID,MIFD
c	write(*, '( i5, z10, i5)' ) iblid, ip, nwr
      IF (IBLID.NE.2) GO TO 910
      IF (IPATT.LT.0 .OR. IPATT.GT.1) GO TO 940
c	write(*,*) ' ipatt', ipatt
c	write(*,*) ' RECORD 3333333'
c      READ (LUN,ERR=900) IBLID,IP,NWR
c	write(*, '( i5, a4, i5)' ) iblid, ip, nwr
      RETURN
C
C   Errors
C
  900 IERROR        = 1
      WRITE (LUN6,2000)
      RETURN
C
  910 IERROR        = 2
      WRITE (LUN6,2100) IBLID
      RETURN
C
  920 IERROR        = 3
      WRITE (LUN6,2200) IFT
      RETURN
C
  930 IERROR       = 4
      WRITE (LUN6,2300)  (CELL(I),I=4,6)
      RETURN
C
  940 IERROR       = 5
      WRITE (LUN6,2400)  IPATT
      RETURN
C
 2000 FORMAT (' MAP> ERROR reading map header record')
 2100 FORMAT (' MAP> ERROR reading map block-id  = ',I5)
 2200 FORMAT (' MAP> ERROR in map file type = ',I5)
 2300 FORMAT (' MAP> ERROR in map cosine(s) out of range:', 3F15.6)
 2400 FORMAT (' MAP> ERROR in map IPATT type = ',I5)
      END

      SUBROUTINE GETR5 (LUN,MIFD,NSIZE,RHO,RHORNG,NRHO,IBP0,IERROR)
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   MAPIN subroutine GETR5 is used to read in the Protein style file type 5 
C   data records.
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( MAXBUF=10000,  LUN6 = 6 )
      REAL      RHO(NSIZE), RHORNG(6), BUF(MAXBUF)
      INTEGER*2 IP, NB, I2BUF(2*MAXBUF)
      INTEGER   I4BUF(1000), MIFD(9), BLID
      EQUIVALENCE (BUF(1),I4BUF(1),I2BUF(1))
C
      IERROR    = 0
      NLINES    = MIFD(2) - MIFD(1) + 1
      LENLIN    = MIFD(5) - MIFD(4) + 1
      NWLINE    = INT( (LENLIN + 1)/2 ) + 3
      IRHO      = 0
      ILINE     = 0
C
C   Loop back to here if several blocks for the layer
C
  100 IF(IBP0.LE.0) THEN
      READ (LUN,ERR=900,IOSTAT=IOS) BLID,IP,NB,(BUF(IB),IB=1,NB)
c	write(*,*) ' blid, ip, nb', blid, ip, nb, i4buf(1), i4buf(2)
      IF (BLID.NE.5) GO TO 910
      IF (NB.LT.4 .OR. NB.GT.MAXBUF) GO TO 920
      IBP0      = 0
      END IF
C
      IF (ILINE.LE.0) THEN
       NWR       = I4BUF(IBP0 + 1)
       IF (NWR.NE.3) GOTO 930
       NLAY      = I4BUF(IBP0 + 2)
       LINE      = I4BUF(IBP0 + 3)
       IF (LINE.NE.NLINES) GOTO 940
       LNTH      = I4BUF(IBP0 + 4)
       IF (LNTH.NE.LENLIN)GO TO 950
       IBP0      = IBP0 + 4
       ILINE     = 1
c	write(*,*) ' nwr nlay line lnth ', nwr, nlay, line, lnth
      END IF
C
      IBP       = IBP0
  110 IF (ILINE.LE.NLINES .AND. IBP + NWLINE.LE.NB) THEN
c	write(*,*) ' ilene, nlines, ibp, nwline, nb',iline, nlines, ibp, 
c     * nwline, nb
       NWR       = I4BUF(IBP + 1)
       TMIN      = BUF(IBP + 2)
       TMAX      = BUF(IBP + 3)
       IF (NWR.NE.NWLINE-1) GO TO 960
       TMID      = 0.5*(TMIN + TMAX)
       TRAT      = (TMAX - TMIN)/65536.
       IBPDEN    = 2 * (IBP + 3)
c	write(*,*) ' iline min max, lnth', iline, tmin, tmax, lnth
C
       DO 120 IL  = 1,LNTH
        IRHO      = IRHO + 1
        IBPDEN    = IBPDEN + 1
        R         = TMID + TRAT*I2BUF(IBPDEN)
        RHO(IRHO) = R
        RSQ       = R * R
        RHORNG(1) = AMIN1(RHORNG(1),R)
        RHORNG(2) = AMAX1(RHORNG(2),R)
        RHORNG(3) = RHORNG(3) + R
        RHORNG(4) = RHORNG(4) + RSQ
        RHORNG(5) = RHORNG(5) + R*RSQ
        RHORNG(6) = RHORNG(6) + RSQ*RSQ
  120  CONTINUE
       IBP       = IBP + NWLINE
       ILINE     = ILINE + 1
c	write(*,*) ' GOTO 110 iline ibp, nwline', iline, ibp, nwline
       GO TO 110 
      END IF
C
      IBP0      = IBP
      IF (IBP0.GE.NB) IBP0 = 0
      IF (ILINE.LE.NLINES) GO TO 100
      NRHO      = NRHO + IRHO
      RETURN
C
C   Errors
C
  900 IERROR   = 1
      WRITE (LUN6,2000) IOS
      RETURN
C
  910 IERROR   = 2
      WRITE (LUN6,2100) BLID
      RETURN
C
  920 IERROR   = 7
      WRITE (LUN6,2200) NB, MAXBUF
      RETURN
C
  930 IERROR   = 8
      WRITE (LUN6,2300) NWR
      RETURN
C
  940 IERROR   = 9
      WRITE (LUN6,2400) LINE, NLINES
      RETURN
C
  950 IERROR   = 10
      WRITE (LUN6,2500) LNTH,LENLIN
      RETURN
C
  960 IERROR   = 11
      WRITE (LUN6,2600) NWR, NWLINE-1
      RETURN
C
 2000 FORMAT (' MAP> Error reading R5 record (IOSTAT=',I5,')')
 2100 FORMAT (' MAP> Record R5 block-id = ',I5,' (should be 5)')
 2200 FORMAT (' MAP> Value of NB in record R5 =',I8,' <4 or >',I5)
 2300 FORMAT (' MAP> Value of NWR in record R5 = ',I12,' (should be 3)')
 2400 FORMAT (' MAP> Value of LINE in record R5 =',I12,' not', I5)
 2500 FORMAT (' MAP> Value of LENGTH in record R5 =',I12,' not', I5)
 2600 FORMAT (' MAP> Value of NWR in record R5 = ',I12,' not',I5)
      END

      SUBROUTINE XCSTAT( rhodat )
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   MAPIN subroutine XCSTAT is used to calculate the map statistics
C   for the electron density map being contoured by program CONTOR
C   of MAPAGE.
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( LUN6 = 6 )
      COMMON /MAPIN2/ RHORNG(6), NRHO
      COMMON /MAPIN3/ PROD, IPLUS, IHIST(256)
      REAL FRQCUT(11,2), RHODAT(4)
      DATA FRQCUT/.005,.01,.05,.1,.3,.5,.7,.9,.95,.99,.995,11*0.0/
C
      IF (NRHO.LE.1) GOTO 900
      IF (RHORNG(1).EQ.RHORNG(2)) GOTO 910
C
      RHODAT(1) = RHORNG(3) / NRHO
      RHODAT(2) = SQRT( (RHORNG(4) - RHODAT(1)**2) / (NRHO-1))
      RHODAT(3) = RHORNG(5) / (NRHO*(RHODAT(2)**2))
      RHODAT(4) = RHORNG(6) / (NRHO*(RHODAT(2)**4))
      WRITE (LUN6,1000) RHORNG(1),RHORNG(2),RHODAT,NRHO
C
C   Histogram stored in (IHIST) presented as frequency intervals
C
      ISUM      = 0
      RAT       = 0.0
      I         = 1
      DO 110 IH = 1,256
      ISUM      = ISUM + IHIST(IH)
      RATL      = RAT
      RAT       = (1.0*ISUM) / NRHO
C
C   Loop while crossing frequency cuts
C
  100 IF (RAT.NE.RATL .AND. RAT.GE.FRQCUT(I,1)) THEN
       RHOI      = (IH-1-IPLUS) / PROD
       RHOL      = (IH-2-IPLUS) / PROD
       FRQCUT(I,2) = RHOL + (RHOI-RHOL)*((FRQCUT(I,1)-RATL)/(RAT-RATL))
       I         = I + 1
       IF (I.EQ.12) GOTO 120
       GOTO 100
      END IF
  110 CONTINUE
  120 WRITE (LUN6,1100) ((FRQCUT(I,J),I=1,11),J=1,2)
      RETURN
C
  900 WRITE (LUN6,2000) 
      RETURN
C
  910 WRITE (LUN6,2100) 
      RETURN
C
 1000 FORMAT ('0MAP> Density statistics : ',/,
     1 '0MAP> Minimum value =',F10.2,' Maximum value =',F10.2,/,
     2 ' MAP> Average value =',F10.2,' Std deviation =',F10.2,/,
     3 ' MAP> Skew value =   ',F10.2,' Kurtosis value=',F10.2,/,
     1 ' MAP> Number Points =',I10)
 1100 FORMAT ('0MAP> Distribution of RHO :'//2X,11F7.3/2X,11F7.1)
 2000 FORMAT ('0MAP> ERROR: No data points for density statistics')
 2100 FORMAT ('0MAP> ERROR: Map has single value')
      END
