#!/usr/bin/env python3
# This reads the map/mrc header information, and dumps it in
# a reasonably friendly way.

# Run as "python dump_header.py foo.mrc"

# This is an expanded version of a script posted on the BB 
# by Christoph Best of EBI

# Martyn Winn Sept 2012
# DT Nov 2018 adapted to python3 

import sys
import array, os
import struct
import numpy as np

verbose = True

filename=sys.argv[1]

print (' ')
print ('Summary of header information from file %s' % filename )
print (' ' )

#header=file(filename).read(1344)
#header=file(filename).read() 
header = array.array( "i" )
f = open( filename, 'br')
header.fromfile(f, os.stat( filename).st_size // header.itemsize )
#header.byteswap()


if (verbose):
  print ('   NX,NY,NZ = ',struct.unpack_from('iii',header,0) )
#  print ('   NX,NY,NZ = ', header[0], header[1], header[2] )
nmm = header[0] * header[1] * header[2]
file_size = os.stat( filename).st_size
file_expect = nmm * 4 + 1024

print ( 'actual file size:', file_size, ' expected:', file_expect, ' difference:', file_size - file_expect, ' bytes')
mode = struct.unpack_from('i',header,3*4)[0]
# exit()
if (verbose):
  print ('   MODE = ',mode )
if (mode == 0):
  print ('File contains signed 8-bit bytes (range -128 to 127)' )
elif (mode == 1):
  print ('File contains signed 2-byte integers' )
elif (mode == 2):
  print ('File contains 4-byte reals' )
elif (mode == 3):
  print ('File contains complex 16-bit integers' )
elif (mode == 4):
  print ('File contains complex 32-bit reals' )
elif (mode == 5):
  print ('File is mode 5: we thought this was never used!' )
elif (mode == 6):
  print ('File is mode 6 (non-standard): contains unsigned 2-byte integers' )
elif (mode == 7):
  print ('File is mode 7 (non-standard): contains signed 4-byte integer' )

if (verbose):
  print ('   NXSTART,NYSTART,NZSTART = ',struct.unpack_from('iii',header,4*4) )
  print ('   MX,MY,MZ = ',struct.unpack_from('iii',header,7*4) )
  print ('   A,B,C = ',struct.unpack_from('fff',header,10*4) )
  print ('   AL,BE,GA = ',struct.unpack_from('fff',header,13*4) )

  print ('   MAPC,MAPR,MAPS = ',struct.unpack_from('iii',header,16*4) )
  print ('   DMIN,DMAX,DMEAN = ',struct.unpack_from('fff',header,19*4) )

ispg = struct.unpack_from('i',header,22*4)[0]
nsymbt = struct.unpack_from('i',header,23*4)[0]
if (verbose):
  print ('   ISPG = ',struct.unpack_from('i',header,22*4) )
  print ('   NSYMBT = ',struct.unpack_from('i',header,23*4) )
if (nsymbt>0):
  print ('There is an extended header of %d bytes' % nsymbt )
  print ('This is expected to consist of %d symmetry record(s)' % (nsymbt/80) )

if (verbose):
  print ('   (extra) = ',struct.unpack_from('25i',header,24*4) )
  print ('   ORIGIN = ',struct.unpack_from('fff',header,49*4) )
  print ('   MAP = ',struct.unpack_from('cccc',header,52*4),'[should be (\'M\',\'A\',\'P\',\' \')]' )
  print ('   MACHST = ','%x' % struct.unpack_from('i',header,53*4),'[should be 4144 or 4444]' )
  print ('   RMS = ',struct.unpack_from('f',header,54*4) )

nlabel = struct.unpack_from('i',header,55*4)[0]
# if (verbose):
#   print ('   NLABL = ',struct.unpack_from('i',header,55*4) )
#   for i in range(nlabel):
#      print ('   label %d = ' % i,struct.unpack_from('80s',header,56*4 + i*80) )

print ('map size =', len(header) - 256*4 - nsymbt )
if (nsymbt>0):
  print ('The following symmetry records are present:' )
  for i in range(nsymbt/80):
    offset = 256*4 + i*80
    print ( struct.unpack_from('80s',header,offset)[0] )
else:
  print( 'no symm present' )

print ('The following should be the first few image/map values:' )
print ( struct.unpack_from('20f',header,256*4) )

print ('The following should be the last 100 image/map values:', len(header)*4 )
print ( struct.unpack_from('100f',header, len(header)*4 - 300*4 ) )
#print ( struct.unpack_from('100f',header, nmm*4+1024 - 100*4 ) )

#print( 'len:', len(header) )

# exit
# 
# igrid = header[7:10]
# ibox = header[4:7] + header[0:3]
# print ( "igrid", igrid, " box", ibox )
# cell_data = struct.unpack_from('fff',header,10*4) 
# cell_data = cell_data + struct.unpack_from('fff',header,13*4) 
# print ( "cell:", cell_data)
# print ( "nmm", nmm )
# 
# new_dens = np.array(header[256:],dtype=float)
# print ( new_dens.shape )
# my_map = 2
# scale_it = 1.0
# main_py.main_put_map(my_map, ibox, igrid, cell_data, nmm, new_dens, scale_it)
# 
# 
# main_py.do_command("show map")
# main_py.do_command("write over file bragg_dens.xmap map 1 xplor")
# main_py.do_command("write over file phased_dens.mrc map 2 mrc")
# main_py.do_command("show map")


if ( len(sys.argv) > 2 ):
  print ( sys.argv, len(sys.argv) )
  if ( file_size - file_expect > 0 ):
    print( ' truncating mrc map file to expected size', file_expect, ' >', sys.argv[2], '<')
    my_new_file = sys.argv[2]
    f = open(my_new_file, 'w+b')
    size = int(file_expect / 4 )
    f.write( header[0:size] )
    f.close()

