      PROGRAM MAPAGE
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   Program MAPAGE is used to read in an electron density map and produce a 
C   DSN6 style electron density map for the on-the-fly contouring routine in 
C   program PS300 FRODO.
C
C   The input Fourier map of real-space densities, sampled on a grid aligned 
C   with the crystal unit cell axes, is converted into the particular direct
C   access form used by PS300 FRODO as a DSN6 input file. FRODO uses this 
C   file under its MAP option. 
C
C   MAPAGE proceeds in two passes through the input file.  It first reads 
C   through the file to obtain the values of PROD and IPLUS.  These values are 
C   used to scale the input electron density map values into byte elements 
C   between 0 and 255.  It determines the appropriate values of PROD and IPLUS
C   to take full advantage of the 0-255 dynamic range of the file. 
C
C   As written, the program requires the input electron density map to be
C   in Steigemann's PROTEIN Style map file with format-5 records. To use 
C   MAPAGE with other electron density map formats than found in Steigemann's 
C   PROTEIN program requires changing subroutine MAPIN.
C
C   The original VAX version written by Alywn Jones (27-JUN-81) and modified
C   modified by Bruce Bush (Merck & CO)  (17-NOV-82).
C
C   Changes for version 6.3:
C
C   Save the input and output file names, and print them in the listing file.
C   Give error message if unable to open input file
C
C   Correct a problem that occured when using a map with an odd number of
C   points in a row. 
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER (MAXRHO = 32000 )
      PARAMETER ( LUN1 = 1,  LUN2 = 2,  LUN5 = 5,  LUN6 = 6 )
	parameter (nrmaps=7000000)
	parameter (nbmaps=80)
	dimension  bmaps(nbmaps), rmaps(nrmaps)
	character*80 line
	character*1 bmaps
	dimension rhodat(4), index(3)
	real*8 dsigma
      COMMON /MAPIN1/ IOXYZ(3), NXYZ(3), IUVW(3),  IGRID(3), CELL(6)
      COMMON /MAPIN2/ RHORNG(6), NRHO 
      COMMON /MAPIN3/ PROD,  IPLUS, IHIST(256)
	common /symm/ nsymm, rts(12,60)
      CHARACTER*30 FILEI, FILEO
      INTEGER      FIRST(256), SLICE(8,MAXRHO), RECORD(512) ,IXYZ(3)
      INTEGER      NBXYZ(3), NUVW(3), NBUVW(3), IBXYZ(3)
      REAL         RHO(MAXRHO) 
C
      WRITE (LUN6,1000)
C
C   Open the input electron density file 
C
c	lun1 = 1
	ires = 2
	write(*,*) ' PROTEIN MAP FILE NAME:'
	read(*,99,err=900) (line(j:j),j=1,80)
99	format( 80a1 )
	filei(1:30) = line(1:30)
	open ( unit = lun1, file = line, shared, readonly,
     * form='unformatted',maxrec=2044, status = 'old', err = 900 )
c      WRITE (LUN6,1100)
c      READ  (LUN5,1200) FILEI
c      CALL ADDEXT (FILEI,'FFT')
c      CALL FOPEN  (LUN1,FILEI,5,.FALSE.,IERR)
c      IF (IERR.NE.0) GOTO 900
C
C   Open the output random access MAP file
C
	write(*,*) ' MAIN MAP FILE NAME:'
	read(*,99,err=900) (line(j:j),j=1,80)
	open ( unit = ires, file = line, status = 'new',
     * carriagecontrol = 'list', err = 910 )
c      WRITE (LUN6,1300)
c      READ  (LUN5,1200) FILEO
c      CALL ADDEXT (FILEO,'DN6')
c      CALL FOPEN  (LUN2,FILEO,4,.TRUE.,IERR)
c      IF (IERR.NE.0) GOTO 910
C
C   Write header for output listing
C
      WRITE (LUN6,1400) 
      WRITE (LUN6,1500) FILEI
      WRITE (LUN6,1600) FILEO
C
C   Read in map header 
C
      CALL MAPIN (.TRUE.,1,RHO,1,IZ,IERR)
      IF (IERR.NE.0) STOP
      WRITE (LUN6,1700) CELL,IOXYZ,NXYZ,IGRID,IUVW
C
C   Number of points along U, V, and W
C
      NUVW(1)   = NXYZ(IUVW(1))
      NUVW(2)   = NXYZ(IUVW(2))
      NUVW(3)   = NXYZ(IUVW(3))
C
C   Number of bricks along U, V, and W
C
      NBUVW(1)  = (NUVW(1)+7)/8
      NBUVW(2)  = (NUVW(2)+7)/8
      NBUVW(3)  = (NUVW(3)+7)/8
C
C   Number of Bricks along X, Y, and Z
C
      NBXYZ(1)  = (NXYZ(1)+7)/8
      NBXYZ(2)  = (NXYZ(2)+7)/8
      NBXYZ(3)  = (NXYZ(3)+7)/8
      WRITE (LUN6,1800) NBXYZ
C
C   Number of points in a map section
C
      LAYER     = NUVW(1) * NUVW(2)
      IF (LAYER.GT.MAXRHO) GOTO 920
C
C   Read through map to calculate scale factors
C
	do i = 1,3
	if ( iuvw(i) .eq. 1 ) ixyz(1) = i
	if ( iuvw(i) .eq. 2 ) ixyz(2) = i
	if ( iuvw(i) .eq. 3 ) ixyz(3) = i
	end do
c
	write(*,*) ' nuvw', nuvw
	do 100 i  = 1, nuvw(3)
	CALL MAPIN (.FALSE.,1,rho,MAXRHO,IZ,IERR)
	IF (IERR.NE.0) GOTO 930
	index(3) = i
	do 5 k = 1, nuvw(2)
	index(2) = k
	do 5 j = 1, nuvw(1)
	index(1) = j
	nn = index(iuvw(1)) + nxyz(1)*(index(iuvw(2))-1) +
     * nxyz(2)*nxyz(1)*(index(iuvw(3))-1)
	rmaps(nn) = rho( (k-1)*nuvw(1) + j)
5	continue
100	continue
c  
	nmaps = layer * nuvw(3)
	rmin =  99999999.9
	rmax = -99999999.9
	aver = 0.0
	do 2 i = 1, nmaps
	aver = aver + rmaps(i)/float( nmaps)
	if ( rmaps(i) .gt. rmax ) rmax = rmaps(i)
	if ( rmaps(i) .lt. rmin ) rmin = rmaps(i)
2	continue
	delta = ( rmax - rmin ) / 255
	write(*,*)  ' MINIMUM, MAXIMUM AND DIFFERENCE STEP IN DENSITY ', 
     * rmin, rmax, delta
c
	dsigma = 0.0
	do 4 i = 1, nmaps
	rms = aver - rmaps(i)
	dsigma = dsigma + rms*rms
4	continue
	sigma = dsqrt(dsigma/ dfloat(nmaps-1))
	write(*,*) ' AVERAGE: ', aver, '  SIGMA: ', sigma
	open ( unit = ires, file = line, status = 'new',
     * carriagecontrol = 'list', err = 104 )
c
12	format( 6e12.5 )
111	format( 10i5 )
	write(ires,12) cell
	write(ires, 111) igrid, ioxyz, nxyz, nsymm
	if ( nsymm .gt. 0 ) then
	write(ires, 89) (j,((rts(3*k+i,j),k=1,3),i=1,3), 
     * (rts(i,j),i=1,3), j=1,nsymm)
c	write(ires, 89) (j,(rts(i,j),i=1,12), j=1,nsymm)
89	format( i5, 3f10.3,/,5x, 3f10.3,/,5x,3f10.3,5x,3f10.3)
	end if
	write(ires,12) rmin/sigma, rmax/sigma, delta/sigma
c	write(0,*) 'i, inte, del, del/delta, ichar(bmaps(i)), '
	ii = 0
	write(ires, '(20a4)', err=104) (rmaps(j)/sigma, j=1, nmaps)
c	if ( ii .gt. 0 ) write(ires, 44, err=104) (bmaps(i), i=1, ii)
	stop
104	write(*,*) ' ERROR WRITEING MAIN DENSITY FILE'
	stop
C
C   Errors
C
  900 WRITE (LUN6,2000) FILEI
      STOP
C
  910 WRITE (LUN6,2100) FILEO
      STOP
C
  920 WRITE (LUN6,2200) (NUVW(I),I=1,2), LAYER, MAXRHO
      STOP
C
  930 WRITE (LUN6,2300) IERR, I
      STOP
C
 1000 FORMAT ('0MAP> Program MAPAGE (PS300 FRODO version 6.3)',//,
     1        ' MAP> RING 3-D map program for on-the-fly contouring')
 1100 FORMAT ('$MAP> Enter the name of the input Fourier file : ')
 1200 FORMAT (A)
 1300 FORMAT ('$MAP> Enter the name of the output DSN6 file : ')
C
 1400 FORMAT ('0MAP> Program MAPAGE (PS300 FRODO version 6.3)')
 1500 FORMAT ('0MAP> Electron density from Fourier file : ',A)
 1600 FORMAT ('0MAP> Electron density written to DSN6 file : ',A)
 1700 FORMAT ('0MAP> Fourier parameters as read from the data-set'//
     1        ' MAP> Cell : ',6F10.5,/,' MAP> Origin : ',5X,3I5,/,
     2        ' MAP> Extent : ',5X,3I5,/, ' MAP> Grid :   ',5X,3I5,/,
     3        ' MAP> U=XYZ(',I1,')   V=XYZ(',I1,')    W=XYZ(',I1,')')
 1800 FORMAT (' MAP> Number of pages along X=',I3,'   Y=',I3,'  Z=',I3)
 1900 FORMAT (' MAP> Input map values scaled to range of 0-255 by',/,
     1        ' MAP> New value = Old value * ',E10.3,' + ',I10)
 2000 FORMAT (' MAP> ERROR opening input file ',A)
 2100 FORMAT (' MAP> ERROR opening output file ',A)
 2200 FORMAT (' MAP> Layer is ',I6,' by ',I6,'=',I10,' total points',/,
     1        ' MAP> Maximum limit is ',I6,' points')
 2300 FORMAT (' MAP> ERROR ',I2,' reading map file on layer',I5)
c      RHODIF    = RHORNG(2)-RHORNG(1)
c      PROD      = 0.0
c      IF (RHODIF.GT.0.0) PROD = 250./RHODIF
c      IPLUS     = 3 - (PROD*RHORNG(1))
c      WRITE (LUN6,1900) PROD,IPLUS
cC
cC   Write output file header block 
cC
cc      CALL PUTDN6 (LUN2,1,FIRST,IERR)
c      IF (IERR.NE.0) STOP
cC
cC   Clear SLICE array
cC
c      DO 110 J = 1,MAXRHO
c      DO 110 I = 1,8
c      SLICE(I,J) = 0.0
c  110 CONTINUE
cC
cC   Reopen the map file      
cC
c      CALL MAPIN (.TRUE., 1, RHO, MAXRHO, IZ, IERR)
c      IF (IERR.NE.0) GOTO 930
cC
cC   Read in 8 layers at a time and send scaled values to output file
cC
c      IBXYZ(IUVW(3)) = 0
c      DO 150 IWBOX = 1,NUVW(3)
c      I1        = MOD(IWBOX-1,8)+1
cC
c      CALL MAPIN(.FALSE.,1,RHO,MAXRHO,IZ,IERR)
c      IF (IERR.NE.0) GOTO 930
c      DO 120 IUV = 1, LAYER
c      IVALUE    = RHO(IUV)*PROD+IPLUS
c      IF (IVALUE.GT.255) IVALUE    = 255
c      IF (IVALUE.LT.  0) IVALUE    = 0
c      IHIST(IVALUE+1) = IHIST(IVALUE+1) + 1
c      SLICE(I1,IUV)   = IVALUE
c  120 CONTINUE
cC
cC   After every eight levels, write output record
cC
c      IF (I1 .EQ. 8 .OR. IWBOX .EQ. NUVW(3)) THEN
cC
c      IF (I1.NE.8) THEN
c       DO 125 IUV = 1, LAYER
c       DO 125 I2  = I1+1,8
c       SLICE(I2,IUV)   = IPLUS
c  125  CONTINUE
c      END IF
cC
cC   Break into 8 x 8 x 8 boxes
cC
c       IBXYZ(IUVW(3)) = IBXYZ(IUVW(3)) + 1
c       DO 140 IVBOX  = 1, NBUVW(2)
c       IBXYZ(IUVW(2)) = IVBOX
c        IVBOX1       = (IVBOX-1) * 8 + 1
c        IVBOX2       = (IVBOX  ) * 8
c       DO 140 IUBOX  = 1, NBUVW(1)
c       IBXYZ(IUVW(1)) = IUBOX
c        IUBOX1       = (IUBOX-1) * 8 + 1
c        IUBOX2       = (IUBOX  ) * 8
cC
cC   Loop over Z, Y, and X points in current box
cC
c      ICT       = 0
c      DO 130 IW = 1,8
c      IXYZ(IUVW(3))   = IW
c      DO 130 IV = IVBOX1, IVBOX2
c      IXYZ(IUVW(2))   = IV - IVBOX1 + 1
c      DO 130 IU = IUBOX1, IUBOX2
c      IXYZ(IUVW(1))   = IU - IUBOX1 + 1
c      ICT       = ICT + 1
c      JCT        = IXYZ(1) + (IXYZ(2)-1)*8 + (IXYZ(3)-1)*64
cC
cC   If either direction over edge,pack record
cC
c      IF ( IU .GT. NUVW(1) .OR. IV .GT. NUVW(2) ) THEN
c       RECORD(JCT) = 0
c      ELSE
c       RECORD(JCT) = SLICE(IW,(IV-1) * NUVW(1) + IU)
c      END IF
c  130 CONTINUE
cC
c      INDEX      =  IBXYZ(1) + (IBXYZ(2)-1)*NBXYZ(1) 
c     1           + (IBXYZ(3)-1)*NBXYZ(1)*NBXYZ(2) + 1
cc      CALL PUTDN6 (LUN2,INDEX,RECORD,IERR)
c      IF (IERR.NE.0) STOP
c  140 CONTINUE
c      END IF
c  150 CONTINUE
cC
cC   Write density statistics:
cC
c      CALL XCSTAT( rhodat )
c	rmin = rhorng(2)/rhodat(2)
c	rmax = rhorng(2)/rhodat(2)
c	delta = (rmax - rmin) / 255
c	write(ires,12) rmin, rmax, delta
c      STOP
      END
      SUBROUTINE MAPIN (DOOPEN,LUN,RHO,NSIZE,NZ,IERROR)
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   Subroutine MAPIN is designed to read in electron density maps from 
C   the Steigeman PROTEIN crystallographic program package having type-5 
C   records.  The routine was originally written by T.A. Jones, completely 
C   modified by B.L. Bush, and converted to INTEGER*4 format by J.S. Sack.
C
C  Variables used :
C
C  DOOPEN  Logical variable determines if the electron density map file
C          should be opened, or if the next layer should be read.
C          if .true., open the fourier map file and read header(s);
C          if .false.,read one section (layer)
C
C  LUN     Integer*4 variable giving the logical unit number of the input 
C          electron density map file.
C
C  NSIZE   Integer*4 variable giving the maximum dimension of variable RHO
C          (number of points in a layer).
C
C  RHO     Real*4 array to be filled with one section's worth of density on 
C          each call to MAPIN (with DOOPEN = .false.).
C
C  NZ      Integer*4 variable set to -1 on return from call with (DOOPEN) and
C          incremented on return from successful read of a section. 
C
C  IERROR  Integer*4 variable which returns 0 on success, nonzero on various 
C          failures. The following error codes are currently in use :
C
C          IERROR = 1  indicates error during read of electron density map
C          IERROR = 2  indicates error in the BLOCK ID for the record
C          IERROR = 3  indicates error in the FILE ID of the R0 record
C          IERROR = 4  indicates error in the unit cell parameters
C          IERROR = 5  indicates error in the map type (IPATT) of the R2 record
C          IERROR = 6  indicates error in the parameters of the R2 record
C          IERROR = 7  indicates error in the buffer size of the R5 record
C          IERROR = 8  indicates error in the number of words per record
C          IERROR = 9  indicates error in the number of lines
C          IERROR = 10 indicates error in the length of each line
C          IERROR = 11 indicates error in the NWR parameter
C
C  Common block MAPIN1 :
C
C  IOXYZ   Integer*4 array of dimension 3 which returns the "origin" of the map
C          (the row, column and layer index of the first entry of the map).
C
C  IMXYZ   Integer*4 array of dimension 3 which returns the "extent" of the map
C          (number of indices in the row, column, and layer directions).
C
C  IUVW    Integer*4 array of dimension 3 containing the orientation of the map
C          with respect to the crystallographic axes.  Most FRODO programs
C          require the map is in layer to layer order. 
C
C  IGRID  Integer*4 array of dimension 3 which returns the map "grid"
C          (number of points along the unit cell).
C
C  CELL    Real*4 array of dimension 6 containing the real space cell
C          cell parameters (in Angstoms and Degrees).
C
C  Common block MAPIN2
C
C  RHORNG  Real*4 array of dimension 6 containing (a) minimum and (b) maximum 
C          density in map file; (c) sum of density entries  sum (r), (d) sum of
C          squared entries sum (r*r), (e) sum of entries to 3d  power, and 
C          (f) sum of entries to 4th power of sum (r*r*r), sum (r*r*r*r). These
C          statistics are initialized on (DOOPEN) calls to +huge,-huge,0,0,0,0;
C          and then updated on later calls.  These values are used by the 
C          calling routines to report statistics at end.
C
C  NRHO    Integer*4 variable used to count the number of points in the map.
C          It is set 0 on (DOOPEN), and then incremented by the number of
C          entries in layer after each read.
C
C  IBP0    Integer*4 variable used internally to keep track of current record
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( LUN6 = 6 )
      COMMON /MAPIN1/ IOXYZ(3), IMXYZ(3), IUVW(3), IGRID(3), CELL(6)
      COMMON /MAPIN2/ RHORNG(6), NRHO, IBP0
      LOGICAL DOOPEN      
      INTEGER MGRID(3), MIFD(9), IPERM(3)
      REAL    RHO(NSIZE)  
C
      IERROR       = 0
C
C   First time through open file and initialize the variables
C
      IF (DOOPEN) THEN
C
      NZ        = -1
      RHORNG(1) = 10000000.0
      RHORNG(2) = -10000000.0
      RHORNG(3) = 0.0
      RHORNG(4) = 0.0
      RHORNG(5) = 0.0
      RHORNG(6) = 0.0
      NRHO      = 0
      IBP0      = 0
C
C   Rewind the input electron density file
C
      REWIND LUN
C
C   Read the PROTEN header records
C
      CALL GET012 ( LUN, CELL, NSYM, IPERM, MGRID, MIFD, IERROR)
      IF (IERROR.NE.0) RETURN
c	write (*,*)  ' CELL ', cell
c	write(*,*)   ' Nsymm', nsym
c	write(*,*) ' iperm', iperm
c	write(*,*) 'mifd ', mifd
c	write (*,*) ' err', ierror
C
C   Load in common block MAPIN1
C
      DO 100 I  = 1,3
       IPT       = (I-1)*3+1
       IOXYZ  (IPERM(I)) = MIFD(IPT)
       IMXYZ  (IPERM(I)) = MIFD(IPT+1)-MIFD(IPT)+1
       IF(MIFD(IPT+2).NE.1) GO TO 900
       IGRID (IPERM(I)) = MGRID(I)
  100 CONTINUE
      IUVW(1)   = IPERM(2)
      IUVW(2)   = IPERM(1)
      IUVW(3)   = IPERM(3)
      RETURN
      END IF
C
C   In each subsequent pass, read one layer of the map
C
      CALL GETR5 (LUN,MIFD,NSIZE,RHO,RHORNG,NRHO,IBP0,IERROR)
      IF (IERROR.NE.0) RETURN
      NZ        = NZ + 1
      RETURN
C
  900 IERROR       = 6
      WRITE (LUN6,2000) (MIFD(I),I=3,9,3)
      RETURN
C
 2000 FORMAT (' MAP> DELTA values are not all 1 (',3I3,')')
      END

      SUBROUTINE GET012 (LUN, CELL, NSYM, IPERM, IGRID, MIFD, IERROR)
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   MAPIN subroutine GET012 is used to read in the Protein style header records
C
C   Original Source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( DEGTOR = 180./3.141593 , LUN6 = 6 )
      INTEGER IPERM(3), IGRID(3), MIFD(9), TITLE(6) 
	integer*2 ip(2)
      REAL    CELL(6)
	common / symm/ nsymm, rts(12,60)
C
C   Read the type zero record
C
      READ (LUN,ERR=900) IBLID,IP,NWR,
     1     (TITLE(I),I=1,4),IFT,(TITLE(I),I=5,6)
c	write(*, '( 4Hirec, i5, 2z10, 2i5)' ) iblid, ip, nwr, ift
	write(*,88) title
88	format( ' TITLE:', 6a4)
      IF (IBLID.NE.0)    GOTO 910
      IF (IFT.NE.3)      GOTO 920
C
C   Read the type 1 record
C
c	write(*,*) ' RECORD 11111111111'
      READ (LUN,ERR=900) IBLID,IP,NWR,CELL,NSYM
c	write(*, '(  i5, z10, i5)' ) iblid, ip, nwr
c	write(*,*) ' nmr', iblid, ip, nwr
	nsymm = nsym
      IF(IBLID.NE.1) GOTO 910
	rewind(unit=lun)
	read(lun) idum
      READ (LUN,ERR=900) IBLID,IP,NWR,CELL,idum,
     * ( (rts(i,j), i=1,12), j=1,nsym)
c	write(*,89) (j,(rts(i,j),i=1,12), j=1,nsym)
c89	format( i5, 3f10.3,/,5x, 3f10.3,/,5x,3f10.3,5x,3f10.3)
C
C  Convert COS(ANGLE) to ANGLE
C
      DO 100 I  = 4,6
      IF ( ABS( CELL(I) ).GT.1.0 ) GO TO 930
      CELL(I) = ACOS(CELL(I)) * DEGTOR
  100 CONTINUE
C
C   Read the type 2 record
C
c	write(*,*) ' RECORD 222222222'
      READ (LUN,ERR=900) IBLID,IP,NWR,IPATT,IPERM,IGRID,MIFD
c	write(*, '( i5, z10, i5)' ) iblid, ip, nwr
      IF (IBLID.NE.2) GO TO 910
      IF (IPATT.LT.0 .OR. IPATT.GT.1) GO TO 940
c	write(*,*) ' ipatt', ipatt
c	write(*,*) ' RECORD 3333333'
c      READ (LUN,ERR=900) IBLID,IP,NWR
c	write(*, '( i5, a4, i5)' ) iblid, ip, nwr
      RETURN
C
C   Errors
C
  900 IERROR        = 1
      WRITE (LUN6,2000)
      RETURN
C
  910 IERROR        = 2
      WRITE (LUN6,2100) IBLID
      RETURN
C
  920 IERROR        = 3
      WRITE (LUN6,2200) IFT
      RETURN
C
  930 IERROR       = 4
      WRITE (LUN6,2300)  (CELL(I),I=4,6)
      RETURN
C
  940 IERROR       = 5
      WRITE (LUN6,2400)  IPATT
      RETURN
C
 2000 FORMAT (' MAP> ERROR reading map header record')
 2100 FORMAT (' MAP> ERROR reading map block-id  = ',I5)
 2200 FORMAT (' MAP> ERROR in map file type = ',I5)
 2300 FORMAT (' MAP> ERROR in map cosine(s) out of range:', 3F15.6)
 2400 FORMAT (' MAP> ERROR in map IPATT type = ',I5)
      END

      SUBROUTINE GETR5 (LUN,MIFD,NSIZE,RHO,RHORNG,NRHO,IBP0,IERROR)
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   MAPIN subroutine GETR5 is used to read in the Protein style file type 5 
C   data records.
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( MAXBUF=10000,  LUN6 = 6 )
      REAL      RHO(NSIZE), RHORNG(6), BUF(MAXBUF)
      INTEGER*2 IP, NB, I2BUF(2*MAXBUF)
      INTEGER   I4BUF(1000), MIFD(9), BLID
      EQUIVALENCE (BUF(1),I4BUF(1),I2BUF(1))
C
      IERROR    = 0
      NLINES    = MIFD(2) - MIFD(1) + 1
      LENLIN    = MIFD(5) - MIFD(4) + 1
      NWLINE    = INT( (LENLIN + 1)/2 ) + 3
      IRHO      = 0
      ILINE     = 0
C
C   Loop back to here if several blocks for the layer
C
  100 IF(IBP0.LE.0) THEN
      READ (LUN,ERR=900,IOSTAT=IOS) BLID,IP,NB,(BUF(IB),IB=1,NB)
c	write(*,*) ' blid, ip, nb', blid, ip, nb, i4buf(1), i4buf(2)
      IF (BLID.NE.5) GO TO 910
      IF (NB.LT.4 .OR. NB.GT.MAXBUF) GO TO 920
      IBP0      = 0
      END IF
C
      IF (ILINE.LE.0) THEN
       NWR       = I4BUF(IBP0 + 1)
       IF (NWR.NE.3) GOTO 930
       NLAY      = I4BUF(IBP0 + 2)
       LINE      = I4BUF(IBP0 + 3)
       IF (LINE.NE.NLINES) GOTO 940
       LNTH      = I4BUF(IBP0 + 4)
       IF (LNTH.NE.LENLIN)GO TO 950
       IBP0      = IBP0 + 4
       ILINE     = 1
c	write(*,*) ' nwr nlay line lnth ', nwr, nlay, line, lnth
      END IF
C
      IBP       = IBP0
  110 IF (ILINE.LE.NLINES .AND. IBP + NWLINE.LE.NB) THEN
c	write(*,*) ' ilene, nlines, ibp, nwline, nb',iline, nlines, ibp, 
c     * nwline, nb
       NWR       = I4BUF(IBP + 1)
       TMIN      = BUF(IBP + 2)
       TMAX      = BUF(IBP + 3)
       IF (NWR.NE.NWLINE-1) GO TO 960
       TMID      = 0.5*(TMIN + TMAX)
       TRAT      = (TMAX - TMIN)/65536.
       IBPDEN    = 2 * (IBP + 3)
c	write(*,*) ' iline min max, lnth', iline, tmin, tmax, lnth
C
       DO 120 IL  = 1,LNTH
        IRHO      = IRHO + 1
        IBPDEN    = IBPDEN + 1
        R         = TMID + TRAT*I2BUF(IBPDEN)
        RHO(IRHO) = R
        RSQ       = R * R
        RHORNG(1) = AMIN1(RHORNG(1),R)
        RHORNG(2) = AMAX1(RHORNG(2),R)
        RHORNG(3) = RHORNG(3) + R
        RHORNG(4) = RHORNG(4) + RSQ
        RHORNG(5) = RHORNG(5) + R*RSQ
        RHORNG(6) = RHORNG(6) + RSQ*RSQ
  120  CONTINUE
       IBP       = IBP + NWLINE
       ILINE     = ILINE + 1
c	write(*,*) ' GOTO 110 iline ibp, nwline', iline, ibp, nwline
       GO TO 110 
      END IF
C
      IBP0      = IBP
      IF (IBP0.GE.NB) IBP0 = 0
      IF (ILINE.LE.NLINES) GO TO 100
      NRHO      = NRHO + IRHO
      RETURN
C
C   Errors
C
  900 IERROR   = 1
      WRITE (LUN6,2000) IOS
      RETURN
C
  910 IERROR   = 2
      WRITE (LUN6,2100) BLID
      RETURN
C
  920 IERROR   = 7
      WRITE (LUN6,2200) NB, MAXBUF
      RETURN
C
  930 IERROR   = 8
      WRITE (LUN6,2300) NWR
      RETURN
C
  940 IERROR   = 9
      WRITE (LUN6,2400) LINE, NLINES
      RETURN
C
  950 IERROR   = 10
      WRITE (LUN6,2500) LNTH,LENLIN
      RETURN
C
  960 IERROR   = 11
      WRITE (LUN6,2600) NWR, NWLINE-1
      RETURN
C
 2000 FORMAT (' MAP> Error reading R5 record (IOSTAT=',I5,')')
 2100 FORMAT (' MAP> Record R5 block-id = ',I5,' (should be 5)')
 2200 FORMAT (' MAP> Value of NB in record R5 =',I8,' <4 or >',I5)
 2300 FORMAT (' MAP> Value of NWR in record R5 = ',I12,' (should be 3)')
 2400 FORMAT (' MAP> Value of LINE in record R5 =',I12,' not', I5)
 2500 FORMAT (' MAP> Value of LENGTH in record R5 =',I12,' not', I5)
 2600 FORMAT (' MAP> Value of NWR in record R5 = ',I12,' not',I5)
      END

      SUBROUTINE XCSTAT( rhodat )
C
C   PS300 FRODO version 6.3  (JSS  01-JAN-1987)
C
C   MAPIN subroutine XCSTAT is used to calculate the map statistics
C   for the electron density map being contoured by program CONTOR
C   of MAPAGE.
C
C   Original source from PS300 FRODO version 5.5  (JWP & MAS  01-JAN-1984)
C
      PARAMETER ( LUN6 = 6 )
      COMMON /MAPIN2/ RHORNG(6), NRHO
      COMMON /MAPIN3/ PROD, IPLUS, IHIST(256)
      REAL FRQCUT(11,2), RHODAT(4)
      DATA FRQCUT/.005,.01,.05,.1,.3,.5,.7,.9,.95,.99,.995,11*0.0/
C
      IF (NRHO.LE.1) GOTO 900
      IF (RHORNG(1).EQ.RHORNG(2)) GOTO 910
C
      RHODAT(1) = RHORNG(3) / NRHO
      RHODAT(2) = SQRT( (RHORNG(4) - RHODAT(1)**2) / (NRHO-1))
      RHODAT(3) = RHORNG(5) / (NRHO*(RHODAT(2)**2))
      RHODAT(4) = RHORNG(6) / (NRHO*(RHODAT(2)**4))
      WRITE (LUN6,1000) RHORNG(1),RHORNG(2),RHODAT,NRHO
C
C   Histogram stored in (IHIST) presented as frequency intervals
C
      ISUM      = 0
      RAT       = 0.0
      I         = 1
      DO 110 IH = 1,256
      ISUM      = ISUM + IHIST(IH)
      RATL      = RAT
      RAT       = (1.0*ISUM) / NRHO
C
C   Loop while crossing frequency cuts
C
  100 IF (RAT.NE.RATL .AND. RAT.GE.FRQCUT(I,1)) THEN
       RHOI      = (IH-1-IPLUS) / PROD
       RHOL      = (IH-2-IPLUS) / PROD
       FRQCUT(I,2) = RHOL + (RHOI-RHOL)*((FRQCUT(I,1)-RATL)/(RAT-RATL))
       I         = I + 1
       IF (I.EQ.12) GOTO 120
       GOTO 100
      END IF
  110 CONTINUE
  120 WRITE (LUN6,1100) ((FRQCUT(I,J),I=1,11),J=1,2)
      RETURN
C
  900 WRITE (LUN6,2000) 
      RETURN
C
  910 WRITE (LUN6,2100) 
      RETURN
C
 1000 FORMAT ('0MAP> Density statistics : ',/,
     1 '0MAP> Minimum value =',F10.2,' Maximum value =',F10.2,/,
     2 ' MAP> Average value =',F10.2,' Std deviation =',F10.2,/,
     3 ' MAP> Skew value =   ',F10.2,' Kurtosis value=',F10.2,/,
     1 ' MAP> Number Points =',I10)
 1100 FORMAT ('0MAP> Distribution of RHO :'//2X,11F7.3/2X,11F7.1)
 2000 FORMAT ('0MAP> ERROR: No data points for density statistics')
 2100 FORMAT ('0MAP> ERROR: Map has single value')
      END
