#!/usr/bin/env python3
# reads ASCII files like Ramachandran plot inputs and writes them in 2D mrc files
import sys
import array, os
import struct
import numpy as np
import math

def get_commands():
    key = ""
    d = dict() 
#    d.clear()
    for arg in sys.argv:
        chars = list(arg)
#        for s in chars:
#            print( s )
#        if ( len (chars) <= 0 ):
#            continue
#        print ( len(chars))
        if ( chars[0] == '-' ):
#            print ( "1- line:%s", arg)
            if( chars[1] == '-' ):
                del chars[0]
                del chars[0]
                key = "".join(chars)
#                print ( "2- key:%s", key )
                d[key] = list ()
            else:
                del chars[0]
                key = "".join(chars)
                d[key] = list ()
#                print ( "1- key:%s", key )
        else:
            if ( key != "" ):
#                print ( "my key:%s -> line %s", key, arg)
                if ( key in d ):
                    d[key].append(arg)
#                    print (d[key])
                else:
                    d[key].append(arg)
#                    print ( key, d[key] )

#    for key, v in d.items():
#        print ( key, v )
    return d

def read_file(my_file):
#    f = gzip.open('file.txt.gz', 'rb')
    file_content = list()
    f = open( my_file, 'r')
    for line in f:
        line.rstrip()
        if ( line[-1] == '\n' ):
            line = line[:-1]
        file_content.append(line)
    f.close()
    return file_content

def write_file( my_file, size, my_header, my_array):
    print("write_file> size", size )
    f = open(my_file, 'w+b')
    f.write( my_header )
    f.write( my_array )
    my_tail = np.zeros(256)
    my_tail = my_tail.astype(np.int32)
    f.write( my_tail )
    f.close()

###################################################################

my_commands = get_commands()
# print ( my_commands )

# print ( "my commands are:", len(my_commands) )
twopi = 2 * math.pi

my_status = { 'in_file': ['__NONE__'],
              'out_file': ['__NONE__.mrc'],
              'format': ['__NONE_'],
              'grid': [180,180,180],
              'cell': [twopi,twopi,twopi, 90, 90, 90],
              'box': [-90,-90, 0, 89, 89, 0],
            }

fl_doit = 'FALSE'
fl_help = 'FALSE'

if ( len( my_commands ) == 0 ): 
    fl_help = 'TRUE'
else:
    for key, v in my_commands.items():
        print ( key, v )
        if ( key == 'i' or key == 'in_file' ):
            my_status['in_file'] = v 
        if ( key == 'o' or key == 'out_file' ):
            my_status['out_file'] = v 
#            my_status['file'].append( v )
        if ( key == 'f' or key == 'format' ):
            my_status['format'] = v
        if ( key == 'h' or key == 'help' ):
            fl_help = 'TRUE'
        if ( key == 'doit' ):
            fl_doit = 'TRUE'

if ( fl_help == "TRUE" ):
    print ( "\n",
"-h| --help)       prints this message with available options and current setup status\n",
"-i| --in_file)    input file name ",  my_status['in_file'], "\n",
"-o| --out_file)   output file name ", my_status['out_file'], "\n",
"-f| --format)     format of input file ", my_status['format'],  "\n",
"-g| --grid)       grid ", my_status['grid'],  "\n",
"-b| --box)        box ",  my_status['box'],  "\n",
"-c| --cell)       cell ", my_status['cell'],  "\n",
"   --doit)           do the conversion\n"
    )
    exit()

my_in_file = my_status['in_file']
my_in_format = my_status['format']
my_out_file = my_status['out_file']

print ( "my inp <", my_in_file, "> out <", my_out_file , "> form <", my_in_format, "> " )

if ( fl_doit == "FALSE" ):
    exit()

my_map = np.zeros( 1 )

my_ii = 0
nm = 180 * 180 * 4
a = np.zeros( nm, dtype = np.float32 )

for my_file in my_in_file:
    my_lines = read_file( my_file )
    print ( my_ii, ' file >', my_file, '<' )
    ii = 0

    for line in my_lines:
        fields = line.split(" " )
        iadr1 = float(fields[1] )
        iadr2 = float(fields[2] )
        iadr = int ( 180 * int ( iadr2 + 179) / 2  + int ( iadr1  + 179 ) / 2 )  + int ( 180*180*my_ii)
        a[iadr] = float( fields[0] )
        #    a[iadr] = float( iadr2 )
        ii = ii + 1
        if ii < 5 : print ( fields, iadr, iadr1, iadr2, "->", a[iadr] )
#        if ( my_ii  == 1 ):
#            my_map = [a]           
#        else:
#            c = np.append( my_map, a )
##            my_map = np.array(my_map)
#            my_map = c
#            np.shape(my_map)
    my_ii = my_ii + 1

my_header = np.zeros(256)
my_header = my_header.astype(np.int32)

#my_header[2] = 2
#my_header[255] = 1
for ii in range(255):
    my_header[ii] = ii
#    print( ii, my_header[ii] )


my_map = np.zeros( nm)
my_map = a
#my_map[0 : 255 ] = my_header


# gids per cell
my_header[0] = 180
my_header[1] = 180
my_header[2] = my_ii

#mode = real*4
my_header[3] = 4

# start of the box
my_header[4] = -89
my_header[5] = -89
my_header[6] = 0

# size of box
my_header[7] = 180
my_header[8] = 180
my_header[9] = my_ii + 1


struct.pack_into('fff', my_header, 10*4, my_status['cell'][0], my_status['cell'][1], my_status['cell'][2] )
struct.pack_into('fff', my_header, 13*4, my_status['cell'][3], my_status['cell'][4], my_status['cell'][5] )

# order
my_header[16] = 1
my_header[17] = 2
my_header[18] = 3

# map kind
#struct.pack_into('cccc', my_header, 52, 'A', '\A', '\A', '\A'  )

# no extension
my_header[23] = 0

# MACHST
#my_header[35] = 4144

#for ii in range(32):
#    print( ii, my_header[ii], my_map[ii] )

#my_size = nm + 256

for my_file in my_out_file:
    file_lines = write_file( my_file, nm, my_header, my_map )

for ii in range(5):
    print( ii, my_map[ii] )

for ii in range(nm-5,nm):
    print( ii, my_map[ii] )

b = my_map.reshape(180,180,my_ii)
np.shape( b )

#print ( b )

