#!/usr/bin/perl -w
use strict;
use lib $ENV{ 'MAIN_CONF'}  ;
use main_config ;

print " @ARGV \n" ;

my %commands = () ;

if ( @ARGV <= 0 ) { push ( @{$commands{'help'}}, ' ' ) ; } 
else { (%commands) = main_config::get_commands ( @ARGV ) ; }

######################################################

my $FILE_INPUT_SEQ = "seq_align.txt" ;
my $FILE_OUTPUT_SEQ = "" ;
my $FILE_MAIN_PAIRS = "seq_pairs" ;
my $FILE_MAIN_COLORS = "seq_colors" ;
my $PROC_ERROR = "FALSE" ;
my $doit = "FALSE" ;
my $SEGM ;
my @reorder = () ;
my @translate = ();
my $K_BARS = 5 ;
my $B_CHARS = 10 ;
my @add_numbers = () ;
my $fl_pairs = "FALSE" ;
my $fl_colors = "FALSE" ;

my $keyword ; 
foreach $keyword ( sort keys (%commands) ) 
{
    print " my $keyword \n";
    if ( $keyword eq 'h' || $keyword eq '?' || substr($keyword, 0,4) eq "help" ) { goto HELP }
    if ( $keyword eq 'i' || substr($keyword, 0,5) eq "input" ) { goto INPUT }
    if ( $keyword eq 'o' || substr($keyword, 0,6) eq "output" ) { goto OUTPUT }
    if ( $keyword eq 'c' || substr($keyword, 0,11) eq "main_colors" ) { goto MAIN_COLORS }
    if ( $keyword eq 'm' || substr($keyword, 0,10) eq "main_pairs" ) { goto MAIN_PAIRS }
    if ( $keyword eq 'r' || substr($keyword, 0,7) eq "reorder" ) { goto REORDER }
    if ( $keyword eq 't' || substr($keyword, 0,9) eq "translate" ) { goto TRANSLATE }
    if ( $keyword eq 'b' || substr($keyword, 0,4) eq "bars" ) { goto BARS }
    if ( $keyword eq 's' || substr($keyword, 0,5) eq "string" ) { goto BCHARS }
    if ( $keyword eq 'a' || substr($keyword, 0,3) eq "add" ) { goto ADD }
    if ( substr($keyword, 0,4) eq "doit" ) { $doit = "TRUE" ; next ; }
    print "unknown option >$keyword<\n" ; $PROC_ERROR = "TRUE" ;

#  -c|--ctab)       file with molecular connectivity table [@{$my_main{"FILE_CTAB"}}]
  HELP:
    print << "END_OF_TEXT" ;
  -h|-help|--help)  prints this message with available options and current status 
  -i|--input)       file with aligned sequences aln/msf/ebi/s3d [$FILE_INPUT_SEQ]
  -o|--output)      file with aligned sequences for publication [$FILE_OUTPUT_SEQ]
  -m|--main_pairs)  macro with MAIN pairing [$FILE_MAIN_PAIRS]
  -c|--main_colors) macro with MAIN colors [$FILE_MAIN_COLORS]
  -r|--reorder)     reorder the sequence list by ids - first id is 0
  -t|--translate)   translate labels: provide pairs from - to
  -b|--bars)        number of string bars in a line [$K_BARS]
  -s|--string)      number of characters in a string in a bar [$B_CHARS]
  -a|--add)         add numbers to the starting sequence - defaults are 0
     --doit)       do the work
END_OF_TEXT
    exit ;


 OUTPUT: $FILE_OUTPUT_SEQ = shift ( @{$commands{$keyword}} ) ; next ;
 INPUT: $FILE_INPUT_SEQ = shift ( @{$commands{$keyword}} ) ; next ;
 MAIN_PAIRS: $FILE_MAIN_PAIRS = shift ( @{$commands{$keyword}} ) ; $fl_pairs = "TRUE" ; next ;
 MAIN_COLORS: $FILE_MAIN_COLORS = shift ( @{$commands{$keyword}} ) ; $fl_colors = "TRUE" ; next ;
 REORDER: @reorder = @{$commands{$keyword}} ; next ;
 TRANSLATE: @translate = @{$commands{$keyword}} ; next ;
 ADD: @add_numbers = @{$commands{$keyword}} ; next ;
 BARS: $K_BARS = shift (@{$commands{$keyword}}) ; next ;
 BCHARS: $B_CHARS = shift (@{$commands{$keyword}}) ; next ;
}

if ( $PROC_ERROR eq "TRUE" ) { exit ; }

#######################################

my $nn = 0 ; my $key ;
foreach $key (sort keys (%commands) ) 
{
    $nn++ ; 
    print "$nn.) $key:  @{$commands{$key}} \n";
}
if ( $doit eq "FALSE" )  { exit ; }

my @id_list ;
my %seq_data ;
my $id ;
my @matrix33 ;
my @tran3 ;

print " file input : $FILE_INPUT_SEQ \n" ;
if ( $FILE_INPUT_SEQ =~ m"msf" ) 
{ 
    read_msf_file ( $FILE_INPUT_SEQ ) ; 
} 
elsif ( $FILE_INPUT_SEQ =~ m"aln" ) 
{ 
    read_clustal_file ( $FILE_INPUT_SEQ ) ; 
}
elsif ( $FILE_INPUT_SEQ =~ m"ebi" ) 
{ 
    read_ebi_file ( $FILE_INPUT_SEQ ) ; 
}
elsif ( lc( $FILE_INPUT_SEQ ) =~ m"s3d" ) 
{ 
    read_promals3d_file ( $FILE_INPUT_SEQ ) ; 
}
else
{
    print " file input other : $FILE_INPUT_SEQ \n" ;
    my ( $ref_seq_data, $ref_id_list, $ref_matrix, $ref_tran3) 
	= main_config::read_jCE_file ( $FILE_INPUT_SEQ ) ; 
    %seq_data = %{$ref_seq_data} ;
    @id_list = @{$ref_id_list} ;
    @matrix33 = @{$ref_matrix} ;
    @tran3 = @{$ref_tran3} ;
}

my $i ;
my @adds = () ;
$i = 0 ; 
while ( $i <= $#id_list )
{
    $adds[$i] = 0 ;
    if ( defined ( $add_numbers[$i] ) ) { $adds[$i] = $add_numbers[$i] ; }
    $i++;
}

if ( exists ( $reorder[0] )  )
{
    my @new_list = () ;
    print $#id_list, "old @id_list order  @reorder\n" ;
    for ( my $ll = 0; $ll <= $#reorder ; $ll++ ) { push(@new_list, $id_list[$reorder[$ll]] ) };
    @id_list = @new_list ;
}
print $#id_list, "new @id_list\n" ;

if ( exists $translate[0] )
{
    for ( my $ll = 0; $ll <= $#translate ; $ll=$ll+2 ) 
    { 
	my $from = $translate[$ll] ;
	my $to = $translate[$ll+1] ;

	print " from $from to $to\n" ;
	my @new_list = @{$seq_data{$from}} ; 
	delete (  $seq_data{$from}) ; 
	@{$seq_data{$to}} = @new_list ;

	
	for ( my $kk = 0; $kk <= $#id_list ; $kk++ ) 
	{ if ( $id_list[$kk] eq $from ) { $id_list[$kk] = $to ; } }
    };
}

# translations

my %pair_list ;
my %seq_data_ids ;
foreach $id ( keys (%seq_data )) 
{ 
    my $data_line = join("", @{$seq_data{$id}} ) ;
    $data_line =~ tr /\./-/ ;
    @{$seq_data{$id}} = split("", $data_line ) ;
#    ${$pair_list{$id}}[0..$#{$seq_data{$id}}] = [-1..-1];
#    ${$seq_data_ids{$id}}[0..$#{$seq_data{$id}}] = [-1..-1];
}

# initialize pair list and ids
foreach $id ( keys (%seq_data )) 
{ 
    my $ll ;
    for ( $ll = 0; $ll <= $#{$seq_data{$id}} ; $ll++ ) { push(@{$pair_list{$id}}, -1) };
    for ( $ll = 0; $ll <= $#{$seq_data{$id}} ; $ll++ ) { push(@{$seq_data_ids{$id}}, -1) };
}




my $j ;
$i = 0 ; 
while ( $i <= $#{$seq_data{$id_list[0]}} )
{
# finding equal to top (.) or equal below (lc) 

    for ( $j = 1; $j <= $#id_list ; $j++ )
    { 
	if ( ${$seq_data{$id_list[$j]}}[$i] eq  ${$seq_data{$id_list[0]}}[$i] &&
	     ${$seq_data{$id_list[0]}}[$i] ne "-" ) 
	{
	    ${$seq_data{$id_list[$j]}}[$i] = "." ; 
	} 
    }
    $i++;
}

# lower identities
$i = 0 ; 
while ( $i <= $#{$seq_data{$id_list[0]}} )
{
    for ( $j = 2; $j <= $#id_list ; $j++ )
    { 
	if ( ${$seq_data{$id_list[$j]}}[$i] eq  ${$seq_data{$id_list[1]}}[$i]  ) 
	{
#	    ${$seq_data{$id_list[1]}}[$i] = lc(${$seq_data{$id_list[1]}}[$i] ) ;
#	    ${$seq_data{$id_list[$j]}}[$i] = lc(${$seq_data{$id_list[$j]}}[$i] ) ;
	} 
    }
    $i++;
}

# homology pairs
my $hh=0 ;
my $dd=0 ;
my $eq=0 ;
$i = 0 ; 
while ( $i <= $#{$seq_data{$id_list[0]}} )
{
    for ( $j = 1; $j <= $#id_list ; $j++ )
    { 
	my $chr_base = uc(${$seq_data{$id_list[0]}}[$i] );
	my $chr_homo = uc(${$seq_data{$id_list[$j]}}[$i] );
	my $lowcase = 0 ;
	if ( $chr_homo eq "\." ) { $eq++ ; } else { $dd++ ; }
	if ( ( $chr_base eq "N" || $chr_base eq "Q"  || $chr_base eq "D"  || $chr_base eq "E" ) &&
	     ( $chr_homo eq "N" || $chr_homo eq "Q"  || $chr_homo eq "D"  || $chr_homo eq "E" ) ) { $lowcase = 1 ; }
	if ( ( $chr_base eq "I" || $chr_base eq "L"  || $chr_base eq "V"  || $chr_base eq "A" ) &&
	     ( $chr_homo eq "I" || $chr_homo eq "L"  || $chr_homo eq "V"  || $chr_homo eq "A" ) ) { $lowcase = 1 ; }
	if ( ( $chr_base eq "K" || $chr_base eq "R" ) &&
	     ( $chr_homo eq "K" || $chr_homo eq "R" ) ) { $lowcase = 1 ; }
	if ( ( $chr_base eq "F" || $chr_base eq "Y" ) &&
	     ( $chr_homo eq "F" || $chr_homo eq "Y" ) ) { $lowcase = 1 ; }
	if ( ( $chr_base eq "S" || $chr_base eq "T" ) &&
	     ( $chr_homo eq "S" || $chr_homo eq "T" ) ) { $lowcase = 1 ; }
	if ( ( $chr_base eq "A" || $chr_base eq "G" ) &&
	     ( $chr_homo eq "A" || $chr_homo eq "G" ) ) { $lowcase = 1 ; }

	if ( $lowcase == 1 ) 
	{
#	    print "$i LOW $lowcase ->$chr_base<>$chr_homo<  \n" ;
	    ${$seq_data{$id_list[$j]}}[$i] = lc(${$seq_data{$id_list[$j]}}[$i] ) ;
	    $hh++ ;
	} 
	else
	{
#	    if ( "ILVA" =~ /\$chr_base/ ) { print "$i HIGH $lowcase ->$chr_base<>$chr_homol<  \n" ; }
	}
#	print "$i A $lowcase ->$chr_base<>$chr_homol<  \n" ; 
    }
    $i++;
}
print " equals: $eq differ $dd   homology $hh  nothomo ", $dd-$hh, "\n" ;


# count dots
my @seq_dots = () ;
my %diffs ;
$i = 0 ;
my $count ;
my $dltion ;
while ( $i <= $#{$seq_data{$id_list[0]}} )
{
    %diffs = () ;
    $count = 0 ;
    $dltion = 0 ;
    $seq_dots[$i] = -1 ;
    my $chr0 =  ${$seq_data{$id_list[0]}}[$i] ;
    my $chr1 ;
    if ( $chr0 eq  '-' ) { $i++; next ; }
    for ( $j = 1; $j <= $#id_list ; $j++ )
    { 
#	if ( ${$seq_data{$id_list[$j]}}[$i] eq  '.' ) { ($count = $count+$j**2) ; } 
	$chr1 =  uc(${$seq_data{$id_list[$j]}}[$i])  ;
	
	if ( $chr1 eq  '.' ) 
	{ $count++ ; $diffs{$chr0} = $count ; }
#	else if ( $chr1 > 153 && $i < 157 ) 
#	{ print $i, " ", $id_list[$j], ">", $chr0, "-", $chr1, "<", $count, "\n" ;  }
	elsif ( $chr1 eq '-' )
	{
	    $dltion++ ;
	}
	else
	{
	    if ( !exists($diffs{$chr1} ) )
	    { $diffs{$chr1} = 1 ; }
	    else
	    { $diffs{$chr1}++ ; }
	}
    }
    $count = $dltion + scalar keys %diffs ;
    
    $seq_dots[$i] = $count ;
#    print $i, "in seq dots $count\n" ;
    $i++;
}

# pair list ids
my $ll;
for ( $j = 0; $j <= $#id_list ; $j++ )
{ 
    $ll = 0 ;
    for ( $i = 0; $i <=$#{$seq_data{$id_list[$j]}} ; $i++ )
    {
	if ( ${$seq_data{$id_list[$j]}}[$i] ne "-" ) 
	{
	    $ll++ ;
	    ${$seq_data_ids{$id_list[$j]}}[$i] = $ll ; ; 
	} 
    } 
}

for ( $i = 0; $i <=$#{$seq_data{$id_list[0]}} ; $i++ )
{
    for ( $j = 1; $j <= $#id_list ; $j++ )
    { 
	if ( ${$seq_data{$id_list[$j]}}[$i] ne "-" &&
	     ${$seq_data{$id_list[0]}}[$i] ne "-" ) 
	{
	    ${$pair_list{$id_list[$j]}}[$i] = $i ;
	} 
    }
}

for ( $i = 0; $i <=$#{$seq_data{$id_list[0]}} ; $i++ )
{
    for ( $j = 1; $j <= $#id_list ; $j++ )
    { 
	if ( ${$seq_data{$id_list[$j]}}[$i] ne "-" &&
	     ${$seq_data{$id_list[0]}}[$i] ne "-" ) 
	{
	    ${$pair_list{$id_list[$j]}}[$i] = $i ;
	} 
    }
}

# find only the filled positions
my @real_seq_list ;
my @real_seq_array ;
my $nc = 0 ;
for ( $i = 0; $i <=$#{$seq_data{$id_list[0]}} ; $i++ )
{
    my $cc = 0 ;
    for ( $j = 0; $j <= $#id_list ; $j++ )
    { 
	if ( ${$seq_data{$id_list[$j]}}[$i] ne "-" )
	{ 
	    $cc++ ;
	}
    } 
    if ($cc > 0 ) 
    { 
	push( @real_seq_list, $i ) ;
	$real_seq_array[$i] = $nc ;
	$nc++ ;
    }
    else { $real_seq_array[$i] = -1 ; }
}

# find borders of print 

my @seq_first = () ;
my @seq_last = () ;

for ( $j = 0; $j <= $#id_list ; $j++ )
{
    $seq_first[$j] = -1 ;
    $seq_last[$j] = -1 ;
}
for ( $i = 0; $i <=$#{$seq_data{$id_list[0]}} ; $i++ )
{
    for ( $j = 0; $j <= $#id_list ; $j++ )
    { 
	if ( ${$seq_data{$id_list[$j]}}[$i] ne "-" )
	{ 
	    if ( $seq_first[$j] < 0 ) { $seq_first[$j] = $i ; }
	    $seq_last[$j] = $i ;
	}
    } 
}

#if ( length( $FILE_OUTPUT_SEQ) > 2 ) { &print_seq_align_table () ; }
&print_seq_align_table () ; 


if ( $fl_colors eq "TRUE" ) { print "SEQ ALIGN> $fl_colors \n"; &seq_colors () ; }
if ( $fl_pairs eq "TRUE" ) { &seq_pair_list () ; }

# fill bars at the beginning to start with 1 in the first line

sub print_seq_align_table
{
    if ( $FILE_OUTPUT_SEQ ne "" ) 
    { open ( OUT,  "> $FILE_OUTPUT_SEQ" ) || die "FILE OPEN ERROR: >$FILE_OUTPUT_SEQ< \n" ; }

my $start ; 
my $start_add ;
my $start_diff ;
my $start_d ;

$start =  $real_seq_array[$seq_first[0]] ;
$start_add = 10 - ( $start - int( $start / 10 )*10-1 );
if ( $start_add == 0 ) { $start_add = 0 ; }
$start_diff = int(( $start + $start_add)/10) ;
$start_d = $K_BARS - ( $start_diff - int($start_diff/$K_BARS)*$K_BARS ) ;

print " start $start add: $start_add  basr: $K_BARS diff $start_diff $start_d \n" ;





#exit ;

#    print $i, "  ", ${$seq_data{$id_list[0]}}[$i], 
#    " = ",  ${$seq_data{$id_list[1]}}[$i], 
#    " = ",  ${$seq_data{$id_list[2]}}[$i], "\n" ;

my $j_line ;
my @line_string = () ;
my $ii; my $kk ;
my @id_count ;
my $real_seq ; my $irq ;
for ( $j_line = 0; $j_line <= $#id_list ; $j_line++ ) { $id_count[$j_line] = 0 ;}

# $i = -$start_add - 10 * $start_d ; 
#$irq = -$start_add - 10 * $start_d ; 

#while ( $i <=  $#{$seq_data{$id_list[0]}} )
$irq = 0 ;
while ( $irq <=  $#real_seq_list )
{
#    $i = $real_seq_list[$irq] ;
#    print " seq data: ", $#{$seq_data{$id_list[0]}}, "\n" ;
#    print " WHILE i=$i\n" ;
    if ( $FILE_OUTPUT_SEQ eq "" ) 
    { print "$irq $start_add $start_d \n" ; }
    else 
    { print OUT "\n" ; }

    for ( $j_line = 0; $j_line <= $#id_list ; $j_line++ )
    { 
	$id = $id_list[$j_line] ;
	my $ids = $id ;
	if ( length($ids) > 15 ) 
	{ $ids = substr($ids, 0, 15) ; }
#	print "i=$i, id=>$id<, j_line=$j_line, \n" ;

#	@{$line_string[$j_line]} = ($id) ;
#	while ( $#{$line_string[$j_line]} < 15 )
#	{ push ( @{$line_string[$j_line]}, " " ) ; }
	my $head = sprintf ( "%15s %4d", $ids, $id_count[$j_line]+1+$adds[$j_line] ) ;
	@{$line_string[$j_line]} = ( $head ) ;
	{ push ( @{$line_string[$j_line]}, " " ) ; }

#	print $i, $id, $j_line, ">", @{$line_string[$j_line]}, "\n" ;
	
	my $j0 = $irq ;
	my $jj ;
	for ( $kk = 0 ; $kk < $K_BARS ; $kk++ )
	{
	    $jj = $j0 + $B_CHARS ;
	    if ( $jj > $#real_seq_list ) { $jj = $#real_seq_list + 1 ; }
#	    print "$kk, $j0, $jj, >", @{$line_string[$j_line]}, "<\n" ;
#	    if ( $jj >  $#{$seq_data{$id_list[0]}} ) { $jj =  $#{$seq_data{$id}}+1 ; }
	    my $ici ;
	    for ( $ici = $j0 ; $ici < $jj; $ici++ ) 
	    {
		if ( $ici < 0 ) { $ii = - 1 ; }
		else { $ii = $real_seq_list[$ici] ; }
		if ( $ii < $seq_first[$j_line] ) 
		{ push ( @{$line_string[$j_line]}, " " ) ; }
		elsif ( $ii > $seq_last[$j_line] ) 
		{ push ( @{$line_string[$j_line]}, " " ) ; }
		else
		{ 
		    if ( ${$seq_data{$id}}[$ii] ne "-" ) { $id_count[$j_line]++ ; }
		    push ( @{$line_string[$j_line]}, ${$seq_data{$id}}[$ii] ) ; 
		}
	    }
	    push ( @{$line_string[$j_line]}, " " ) ; 
	    if ( $jj >=  $#{$seq_data{$id_list[0]}} ) { last ; }
	    $j0 = $jj ; 
	}
    }
    $irq = $irq + $K_BARS * $B_CHARS ;
    for ( $j_line = 0; $j_line <= $#id_list ; $j_line++ )
    { 
	$id = $id_list[$j_line] ;
	if ( $FILE_OUTPUT_SEQ eq "" ) 
	{ print @{$line_string[$j_line]}, "\n" ;}
	else 
	{ print OUT @{$line_string[$j_line]}, "\n" ;}
    }
}
    if ( $FILE_OUTPUT_SEQ eq "" ) 
    { print "\n" ; }
    else 
    { print OUT "\n" ; }
close ( OUT ) ;
}

sub seq_pair_list
{
my $my_t_id = $id_list[0] ; $my_t_id =~ s/ /_/g ;
my $my_id ;
my $MY_FILE ;

for ( $j = 1; $j <= $#id_list ; $j++ )
{

    $my_id = $id_list[$j] ;     $my_id =~ s/ /_/g ;
    $MY_FILE = "make_pairs_from_" . $my_id . "_to_" . $my_t_id . ".com" ;
    open ( MACRO,  "> $MY_FILE" ) || die "FILE OPEN ERROR: >$MY_FILE< \n" ;
    my $str =  $id_list[0]; 
    print MACRO "set vari R0 int resi sele $str end \n" ;
    print MACRO "set vari R0 = R0 - 1\n" ;
    $str = $id_list[$j] ; 
    print MACRO "set vari R2 int resi sele $str end \n" ;
    print MACRO "set vari R2 = R2 - 1\n" ;

    print MACRO "\n" ;
    print MACRO "! fitting \"$id_list[$j]\" \n" ;
    print MACRO "\n" ;
    print MACRO "make pair init \n" ;
    for ( $i = 0; $i < $#{$seq_data{$id_list[$j]}} ; $i++ )
    { 
	my $ll = ${$pair_list{$id_list[$j]}}[$i] ;
#	print " ll=>$ll<\n" ;
	if ( $ll > -1 )
	{
	    my $r_1 = ${$seq_data_ids{$id_list[$j]}}[$i] ;
	    my $r_2 = ${$seq_data_ids{$id_list[0]}}[$ll] ;
	    my $kk = $j+1;
print MACRO << "END_OF_TEXT" ;
make pair sele resi numb R2 + $r_1 .a atom name CA end \\ 
          sele resi numb R0 + $r_2 .a atom name CA end 
END_OF_TEXT
	} 
    }
print MACRO << "END_OF_TEXT" ;
rms coor all pair 
rota atom sele $my_id end rms 
tran atom sele $my_id end rms 

END_OF_TEXT
close ( MACRO ) ;

}
}

sub seq_colors
{
    my $my_id;
    my $MY_FILE ;
$my_id = $id_list[0] ; $my_id =~ s/ /_/g ;
$MY_FILE = "seq_color_" . $my_id . ".com" ;
open ( MACRO,  "> $MY_FILE" ) || die "FILE OPEN ERROR: >$MY_FILE< \n" ;

    my $ATOMS = " .n atom name H* " ;
print MACRO "\n" ;
print MACRO "! coloring \n" ;
print MACRO "key $my_id sele segm name AA end \n" ;
print MACRO "set vari R0 int resi sele $my_id end \n" ;
print MACRO "set vari R0 = R0 + 3\n" ;
print MACRO "vlt \n" ;
print MACRO "color 32 + 1 red 1.0 gre 0.8 blue 0.8 \n" ;
print MACRO "color 32 + 2 red 1.0 gre 0.6 blue 0.6 \n" ;
print MACRO "color 32 + 3 red 1.0 gre 0.3 blue 0.3 \n" ;
print MACRO "color 32 + 4 red 1.0 gre 0.0 blue 0.0 \n" ;
print MACRO "exit \n" ;
print MACRO "set color\n" ;
print MACRO "sele $my_id end = 32\n" ;

    my $II = 0 ;
for ( $i = 0; $i <= $#{$seq_data{$id_list[0]}} ; $i++ )
{ 
    my $ll ;
    $ll = 31 + $seq_dots[$i] ;
#    if ($seq_dots[$i] == 5 ) { $ll = 32 ; }
#    if ($seq_dots[$i] == 4 ) { $ll = 220 ; }
#    if ($seq_dots[$i] == 3 ) { $ll = 200 ; }
#    if ($seq_dots[$i] == 2 ) { $ll = 148 ; }
#    if ($seq_dots[$i] == 1 ) { $ll = 160 ; }
#    if ($seq_dots[$i] == 0 ) { $ll = 128 ; }
#    my $r_1 = + ${$seq_data_ids{$id_list[0]}}[$i] ;
#    if ( $r_1 > 0 )
    if ( $seq_dots[$i] >= 1 )
    {
	$II++ ;
	print MACRO "sele resi numb R0 + $II .a $ATOMS end col $ll ! ", $seq_dots[$i], " ", ${$seq_data{$id_list[0]}}[$i], "\n" ;
    } 
}

print MACRO << "END_OF_TEXT" ;
exit

END_OF_TEXT

close ( MACRO ) ;
}

sub read_msf_file 

{
    my $INPUT_FILE = $_[0] ;
    my $line ;
    my @SEQ_INPUT_LINES ;

    open ( TXT, $INPUT_FILE ) || die "READ FILE OPEN ERROR: >$INPUT_FILE< \n" ; 
    print " read alignment msf file: >$INPUT_FILE<\n" ;
    while(<TXT>)
    {
	chomp ;
	$line = $_ ;
#	print "my_line:", $line, "\n" ;
	push ( @SEQ_INPUT_LINES, $line ) ;
    }
    close(TXT) ;

# print " input lines:", @SEQ_INPUT_LINES, "\n" ;
    my $data_flag = "N" ;
    
    foreach $line ( @SEQ_INPUT_LINES ) 
    {
	my @line_array = split( " ", $line ) ;
#    print $#line_array, " file: each >$line<\n" ;
	if ( $#line_array >= 0 ) 
	{
	    if ( $line_array[0] eq "//" ) 
	    { 
		$data_flag = "Y" ; 
#	    print "YESSSS\n" ;
		next ;
	    } 
#	else 
#	{ 
#	    print "NOOOO  ",  $#line_array, "\n" ; 
#	}
	    
	    if ( $data_flag eq "Y" )
	    {
#	    print "data: >$line< \n" ;
		$id = shift ( @line_array ) ;
		if ( !exists($seq_data{$id} ) ) 
		{ 
		    print "push $id\n" ; 
		    push(@id_list, $id) ; 
		}
#	    print $id, " d: >",@line_array,"< \n" ;
		my @chars = split("", join("", @line_array)) ;
		push ( @{$seq_data{$id}}, @chars ) ;
	    }
	}
    }
    
#return ( \%seq_data ) ;
    
}



sub read_clustal_file 

{
    my $INPUT_FILE = $_[0] ;
    my $line ;
    my @SEQ_INPUT_LINES ;

    open ( TXT, $INPUT_FILE ) || die "READ FILE OPEN ERROR: >$INPUT_FILE< \n" ; 
    print " read alignment file: >$INPUT_FILE<\n" ;
    while(<TXT>)
    {
	chomp ;
	$line = $_ ;
	push ( @SEQ_INPUT_LINES, $line ) ;
    }
    close(TXT) ;

#print @SEQ_INPUT_LINES, "\n" ;
    my $data_flag = "N" ;
    
    foreach $line ( @SEQ_INPUT_LINES ) 
    {
	my @line_array = split( " ", $line ) ;
#    print $#line_array, " file: >$line<\n" ;
	if ( $#line_array >= 0 ) 
	{
	    if ( $line_array[0] eq "CLUSTAL" ) 
	    { 
		$data_flag = "Y" ; 
#	    print "YESSSS\n" ;
		next ;
	    } 
#	else 
#	{ 
#	    print "NOOOO  ",  $#line_array, "\n" ; 
#	}
	    if ( substr($line, 0, 5) eq "     " ) { next ; }
	    
	    if ( $data_flag eq "Y" )
	    {
#		print "data: >$line< \n" ;
		$id = shift ( @line_array ) ;
		if ( !exists($seq_data{$id} ) ) { print "push $id\n" ; push(@id_list, $id) ; }
#		printf "%12s %5s %80s, %3s", $id, " d: >",@line_array,"< \n" ;
		my @chars = split("", join("", @line_array)) ;
		push ( @{$seq_data{$id}}, @chars ) ;
	    }
	}
    }
    
#return ( \%seq_data ) ;
    
}

sub read_ebi_file 

{
    my $INPUT_FILE = $_[0] ;
    my $line ;
    my @SEQ_INPUT_LINES ;

    open ( TXT, $INPUT_FILE ) || die "READ FILE OPEN ERROR: >$INPUT_FILE< \n" ; 
    print " read alignment ebi file: >$INPUT_FILE<\n" ;
    my $nn = 0 ;
    while(<TXT>)
    {
	$nn++;
	chomp ;
	$line = $_ ;
#	print "my_line:", $line, "\n" ;
	push ( @SEQ_INPUT_LINES, $line ) ;
    }
    close(TXT) ;
    print $nn-2, " n of lines\n" ;

    shift ( @SEQ_INPUT_LINES) ;
    shift ( @SEQ_INPUT_LINES) ;
#    print " input lines:", @SEQ_INPUT_LINES, "\n" ;
    my $data_flag = "N" ;
    
    foreach $line ( @SEQ_INPUT_LINES ) 
    {
	my @line_array = split( " ", $line ) ;
#	print $#line_array, " file: each >$line<\n" ;
	my $data_flag = "N" ;
	if ( $#line_array >= 0 ) 
	{
	    if ( $line_array[0] ne " " ) 
	    { 
		$data_flag = "Y" ; 
#		print "YESSSS\n" ;
#		next ;
	    } 
#	else 
#	{ 
#	    print "NOOOO  ",  $#line_array, "\n" ; 
#	}
	    
	    if ( $data_flag eq "Y" )
	    {
#		print "data: >$line< \n" ;
		$id = shift ( @line_array ) ;
		if ( !exists($seq_data{$id} ) ) { print "push $id\n" ; push(@id_list, $id) ; }
#		print $id, " d: >",@line_array,"< \n" ;
		my @chars = split("", join("", @line_array)) ;
		push ( @{$seq_data{$id}}, @chars ) ;
	    }
	}
    }

    my @my_list ;
    @my_list = @id_list ;
    my $base = shift ( @my_list ) ;

    my $lim = $#{$seq_data{$base}} ;
    my $count ;
    foreach $id (@my_list )
    {
#	print " ech $base seg <$id> $lim\n" ;
	for ( $count = 0; $count <= $lim; $count++ )
	{
	    if ( ${$seq_data{$id}}[$count] eq "-" )
	    { ${$seq_data{$id}}[$count] = ${$seq_data{$base}}[$count] ; }
# account for potential editing errors
	    if ( ${$seq_data{$id}}[$count] eq "." )
	    { ${$seq_data{$id}}[$count] = ${$seq_data{$base}}[$count] ; }
	}
    }
    print @{$seq_data{$base}}, "\n" ;
#return ( \%seq_data ) ;
    
}
sub read_promals3d_file 

{
    my $INPUT_FILE = $_[0] ;
    my $line ;
    my @line_array ;
    my @SEQ_INPUT_LINES ;

    open ( TXT, $INPUT_FILE ) || die "READ FILE OPEN ERROR: >$INPUT_FILE< \n" ; 
    print " read alignment file: >$INPUT_FILE<\n" ;
    while(<TXT>)
    {
	chomp ;
	$line = $_ ;
	push ( @SEQ_INPUT_LINES, $line ) ;
    }
    close(TXT) ;

#print @SEQ_INPUT_LINES, "\n" ;
    my $data_flag = "N" ;
    
    $line = shift ( @SEQ_INPUT_LINES ) ;
    @line_array = split( " ", $line ) ;
    if ( $#line_array >= 0 ) 
	{
	    if ( $line_array[1] ne "PROMALS3D" ) 
	    { 
		die "not a PROMALS3D file >$INPUT_FILE< \n" ; 
	    } 
	}

    foreach $line ( @SEQ_INPUT_LINES ) 
    {
    @line_array = split( " ", $line ) ;
#    print $#line_array, " file: >$line<\n" ;
	if ( $#line_array >= 0 ) 
	{
	    if ( substr($line, 0, 5) eq "     " ) { next ; }
	    
	    $id = shift ( @line_array ) ;
	    if ( $id =~ m"\:" ) 
	    { 
		$data_flag = "N" ; 
#		print "info: >$line< \n" ;
	    }
	    else 
	    { 
		$data_flag = "Y" ; 
#		print "data: >$line< \n" ;
	    }
	    


#	    printf "%12s %5s %80s, %3s", $id, " d: >",@line_array,"< \n" ;
	    if ( $data_flag eq "Y" )
	    {
		if ( !exists($seq_data{$id} ) ) 
		{ 
		    print "push $id\n" ; 
		    push(@id_list, $id) ; 
		}
		shift ( @line_array ) ;
		pop ( @line_array ) ;
		my @chars = split("", join("", @line_array)) ;
		push ( @{$seq_data{$id}}, @chars ) ;
#		print "  my  line >$line< \n" ;
	    }
	    else
	    {
		if ( !exists($seq_data{$id} ) ) 
		{ 
		    print "push $id\n" ; 
		    push(@id_list, $id) ; 
		}
		my $s = substr($line, 32, 70 ) ;
		my @chars = split("", $s ) ;
		push ( @{$seq_data{$id}}, @chars ) ;
#		print " else line >$line< \n" ;
	    }
	}
    }
}
